@@ -1,9 +1,11 @@
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
 #ifndef DUNE_ALBERTAGRID_CC
 #define DUNE_ALBERTAGRID_CC
 
 //************************************************************************
 //
-//  implementation of AlbertaGrid 
+//  implementation of AlbertaGrid
 //
 //  namespace Dune
 //
@@ -36,16 +38,16 @@ namespace Dune
 
   template< int dim, int dimworld >
   inline AlbertaGrid < dim, dimworld >::AlbertaGrid ()
-  : mesh_(),
-    maxlevel_( 0 ),
-    numBoundarySegments_( 0 ),
-    hIndexSet_( dofNumbering_ ),
-    idSet_( hIndexSet_ ),
-    levelIndexVec_( (size_t)MAXL, 0 ),
-    leafIndexSet_( 0 ),
-    sizeCache_( *this ),
-    leafMarkerVector_( dofNumbering_ ),
-    levelMarkerVector_( (size_t)MAXL, MarkerVector( dofNumbering_ ) )
+    : mesh_(),
+      maxlevel_( 0 ),
+      numBoundarySegments_( 0 ),
+      hIndexSet_( dofNumbering_ ),
+      idSet_( hIndexSet_ ),
+      levelIndexVec_( (size_t)MAXL, 0 ),
+      leafIndexSet_( 0 ),
+      sizeCache_( *this ),
+      leafMarkerVector_( dofNumbering_ ),
+      levelMarkerVector_( (size_t)MAXL, MarkerVector( dofNumbering_ ) )
   {
     checkAlbertaDimensions< dim, dimworld>();
   }
@@ -54,18 +56,18 @@ namespace Dune
   template< int dim, int dimworld >
   template< class Proj, class Impl >
   inline AlbertaGrid< dim, dimworld >
-    ::AlbertaGrid ( const Alberta::MacroData< dimension> &macroData,
-                    const Alberta::ProjectionFactoryInterface< Proj, Impl > &projectionFactory )
-  : mesh_(),
-    maxlevel_( 0 ),
-    numBoundarySegments_( 0 ),
-    hIndexSet_( dofNumbering_ ),
-    idSet_( hIndexSet_ ),
-    levelIndexVec_( (size_t)MAXL, 0 ),
-    leafIndexSet_ ( 0 ),
-    sizeCache_( *this ),
-    leafMarkerVector_( dofNumbering_ ),
-    levelMarkerVector_( (size_t)MAXL, MarkerVector( dofNumbering_ ) )
+  ::AlbertaGrid ( const Alberta::MacroData< dimension> &macroData,
+                  const Alberta::ProjectionFactoryInterface< Proj, Impl > &projectionFactory )
+    : mesh_(),
+      maxlevel_( 0 ),
+      numBoundarySegments_( 0 ),
+      hIndexSet_( dofNumbering_ ),
+      idSet_( hIndexSet_ ),
+      levelIndexVec_( (size_t)MAXL, 0 ),
+      leafIndexSet_ ( 0 ),
+      sizeCache_( *this ),
+      leafMarkerVector_( dofNumbering_ ),
+      levelMarkerVector_( (size_t)MAXL, MarkerVector( dofNumbering_ ) )
   {
     checkAlbertaDimensions< dim, dimworld >();
 
@@ -82,18 +84,18 @@ namespace Dune
 
   template< int dim, int dimworld >
   inline AlbertaGrid< dim, dimworld >
-    ::AlbertaGrid ( const Alberta::MacroData< dimension> &macroData,
-                    const Dune::shared_ptr< DuneBoundaryProjection< dimensionworld > > &projection )
-  : mesh_(),
-    maxlevel_( 0 ),
-    numBoundarySegments_( 0 ),
-    hIndexSet_( dofNumbering_ ),
-    idSet_( hIndexSet_ ),
-    levelIndexVec_( (size_t)MAXL, 0 ),
-    leafIndexSet_ ( 0 ),
-    sizeCache_( *this ),
-    leafMarkerVector_( dofNumbering_ ),
-    levelMarkerVector_( (size_t)MAXL, MarkerVector( dofNumbering_ ) )
+  ::AlbertaGrid ( const Alberta::MacroData< dimension> &macroData,
+                  const Dune::shared_ptr< DuneBoundaryProjection< dimensionworld > > &projection )
+    : mesh_(),
+      maxlevel_( 0 ),
+      numBoundarySegments_( 0 ),
+      hIndexSet_( dofNumbering_ ),
+      idSet_( hIndexSet_ ),
+      levelIndexVec_( (size_t)MAXL, 0 ),
+      leafIndexSet_ ( 0 ),
+      sizeCache_( *this ),
+      leafMarkerVector_( dofNumbering_ ),
+      levelMarkerVector_( (size_t)MAXL, MarkerVector( dofNumbering_ ) )
   {
     checkAlbertaDimensions< dim, dimworld >();
 
@@ -116,16 +118,16 @@ namespace Dune
 
   template < int dim, int dimworld >
   inline AlbertaGrid< dim, dimworld >
-    ::AlbertaGrid ( const std::string &macroGridFileName )
-  : mesh_(),
-    maxlevel_( 0 ),
-    hIndexSet_( dofNumbering_ ),
-    idSet_( hIndexSet_ ),
-    levelIndexVec_( (size_t)MAXL, 0 ),
-    leafIndexSet_ ( 0 ),
-    sizeCache_( *this ),
-    leafMarkerVector_( dofNumbering_ ),
-    levelMarkerVector_( (size_t)MAXL, MarkerVector( dofNumbering_ ) )
+  ::AlbertaGrid ( const std::string &macroGridFileName )
+    : mesh_(),
+      maxlevel_( 0 ),
+      hIndexSet_( dofNumbering_ ),
+      idSet_( hIndexSet_ ),
+      levelIndexVec_( (size_t)MAXL, 0 ),
+      leafIndexSet_ ( 0 ),
+      sizeCache_( *this ),
+      leafMarkerVector_( dofNumbering_ ),
+      levelMarkerVector_( (size_t)MAXL, MarkerVector( dofNumbering_ ) )
   {
     checkAlbertaDimensions< dim, dimworld >();
 
@@ -134,7 +136,7 @@ namespace Dune
     {
       DUNE_THROW( AlbertaIOError,
                   "Grid file '" << macroGridFileName
-                   << "' is not in ALBERTA macro triangulation format." );
+                                << "' is not in ALBERTA macro triangulation format." );
     }
 
     setup();
@@ -198,7 +200,7 @@ namespace Dune
   template< int dim, int dimworld >
   template< int codim, PartitionIteratorType pitype >
   inline typename AlbertaGrid< dim, dimworld >::Traits
-    ::template Codim< codim >::template Partition<pitype>::LevelIterator
+  ::template Codim< codim >::template Partition<pitype>::LevelIterator
   AlbertaGrid< dim, dimworld >::lbegin ( int level ) const
   {
     typedef AlbertaGridLevelIterator< codim, pitype, const This > LevelIteratorImp;
@@ -218,7 +220,7 @@ namespace Dune
   template< int dim, int dimworld >
   template< int codim, PartitionIteratorType pitype >
   inline typename AlbertaGrid< dim, dimworld >::Traits
-    ::template Codim< codim >::template Partition< pitype >::LevelIterator
+  ::template Codim< codim >::template Partition< pitype >::LevelIterator
   AlbertaGrid < dim, dimworld >::lend ( int level ) const
   {
     typedef AlbertaGridLevelIterator< codim, pitype, const This > LevelIteratorImp;
@@ -231,7 +233,7 @@ namespace Dune
   template< int dim, int dimworld >
   template< int codim >
   inline typename AlbertaGrid< dim, dimworld >::Traits
-    ::template Codim< codim >::LevelIterator
+  ::template Codim< codim >::LevelIterator
   AlbertaGrid < dim, dimworld >::lbegin ( int level ) const
   {
     return lbegin< codim, All_Partition >( level );
@@ -241,7 +243,7 @@ namespace Dune
   template< int dim, int dimworld >
   template< int codim >
   inline typename AlbertaGrid< dim, dimworld >::Traits
-    ::template Codim< codim >::LevelIterator
+  ::template Codim< codim >::LevelIterator
   AlbertaGrid< dim, dimworld >::lend ( int level ) const
   {
     return lend< codim, All_Partition >( level );
@@ -251,11 +253,11 @@ namespace Dune
   template< int dim, int dimworld >
   template< int codim, PartitionIteratorType pitype >
   inline typename AlbertaGrid< dim, dimworld >::Traits
-    ::template Codim< codim >::template Partition< pitype >::LeafIterator
+  ::template Codim< codim >::template Partition< pitype >::LeafIterator
   AlbertaGrid< dim, dimworld >::leafbegin () const
   {
     typedef AlbertaGridLeafIterator< codim, pitype, const This > LeafIteratorImp;
-    
+
     MarkerVector &markerVector = leafMarkerVector_;
     const int firstMarkedCodim = 2;
     if( (codim >= firstMarkedCodim) && !markerVector.up2Date() )
@@ -268,7 +270,7 @@ namespace Dune
   template< int dim, int dimworld >
   template< int codim, PartitionIteratorType pitype >
   inline typename AlbertaGrid< dim, dimworld >::Traits
-    ::template Codim< codim >::template Partition< pitype >::LeafIterator
+  ::template Codim< codim >::template Partition< pitype >::LeafIterator
   AlbertaGrid< dim, dimworld >::leafend () const
   {
     typedef AlbertaGridLeafIterator< codim, pitype, const This > LeafIteratorImp;
@@ -279,7 +281,7 @@ namespace Dune
   template< int dim, int dimworld >
   template< int codim >
   inline typename AlbertaGrid<dim,dimworld>::Traits
-    ::template Codim< codim >::LeafIterator
+  ::template Codim< codim >::LeafIterator
   AlbertaGrid< dim, dimworld >::leafbegin () const
   {
     return leafbegin< codim, All_Partition >();
@@ -289,7 +291,7 @@ namespace Dune
   template< int dim, int dimworld >
   template< int codim >
   inline typename AlbertaGrid< dim, dimworld >::Traits
-    ::template Codim< codim >::LeafIterator
+  ::template Codim< codim >::LeafIterator
   AlbertaGrid < dim, dimworld >::leafend () const
   {
     return leafend< codim, All_Partition >();
@@ -301,12 +303,12 @@ namespace Dune
   {
     typedef typename Traits::template Codim< 0 >::LeafIterator LeafIterator;
 
-    // only MAXL levels allowed 
+    // only MAXL levels allowed
     assert( (refCount >= 0) && (refCount + maxlevel_ < MAXL) );
-    
+
     for( int i = 0; i < refCount; ++i )
     {
-      // mark all interior elements 
+      // mark all interior elements
       const LeafIterator endit = leafend< 0 >();
       for( LeafIterator it = leafbegin< 0 >(); it != endit; ++it )
         mark( 1, *it );
@@ -321,16 +323,16 @@ namespace Dune
   template< int dim, int dimworld >
   template< class DataHandle >
   inline void AlbertaGrid< dim, dimworld >
-    ::globalRefine ( int refCount, AdaptDataHandleInterface< This, DataHandle > &handle )
+  ::globalRefine ( int refCount, AdaptDataHandleInterface< This, DataHandle > &handle )
   {
     typedef typename Traits::template Codim< 0 >::LeafIterator LeafIterator;
 
-    // only MAXL levels allowed 
+    // only MAXL levels allowed
     assert( (refCount >= 0) && (refCount + maxlevel_ < MAXL) );
-    
+
     for( int i = 0; i < refCount; ++i )
     {
-      // mark all interior elements 
+      // mark all interior elements
       const LeafIterator endit = leafend< 0 >();
       for( LeafIterator it = leafbegin< 0 >(); it != endit; ++it )
         mark( 1, *it );
@@ -378,9 +380,9 @@ namespace Dune
 
   template< int dim, int dimworld >
   inline bool AlbertaGrid< dim, dimworld >
-    ::mark( int refCount, const typename Traits::template Codim< 0 >::Entity &e )
+  ::mark( int refCount, const typename Traits::template Codim< 0 >::Entity &e )
   {
-    // if not leaf entity, leave method 
+    // if not leaf entity, leave method
     if( !e.isLeaf() )
       return false;
 
@@ -401,7 +403,7 @@ namespace Dune
 
   template< int dim, int dimworld >
   inline int AlbertaGrid< dim, dimworld >
-    ::getMark( const typename Traits::template Codim< 0 >::Entity &e ) const
+  ::getMark( const typename Traits::template Codim< 0 >::Entity &e ) const
   {
     return getRealImplementation( e ).elementInfo().getMark();
   }
@@ -423,7 +425,7 @@ namespace Dune
     if( refined || coarsened )
       calcExtras();
 
-    // return true if elements were created 
+    // return true if elements were created
     return refined;
   }
 
@@ -431,7 +433,7 @@ namespace Dune
   template< int dim, int dimworld >
   template< class DataHandle >
   inline bool AlbertaGrid < dim, dimworld >
-    ::adapt ( AdaptDataHandleInterface< This, DataHandle > &handle )
+  ::adapt ( AdaptDataHandleInterface< This, DataHandle > &handle )
   {
     // minimum number of elements assumed to be created during adaptation
     const int defaultElementChunk = 100;
@@ -439,10 +441,10 @@ namespace Dune
     preAdapt();
     const int refineMarked = adaptationState_.refineMarked();
     handle.preAdapt( std::max( defaultElementChunk, 4*refineMarked ) );
-    
+
     typedef Alberta::AdaptRestrictProlongHandler
-      < This, AdaptDataHandleInterface< This, DataHandle > >
-      DataHandler;
+    < This, AdaptDataHandleInterface< This, DataHandle > >
+    DataHandler;
     DataHandler dataHandler( *this, handle );
 
     typedef AdaptationCallback< DataHandler > Callback;
@@ -470,7 +472,7 @@ namespace Dune
   template< int dim, int dimworld >
   inline const Alberta::GlobalVector &
   AlbertaGrid< dim, dimworld >
-    ::getCoord ( const ElementInfo &elementInfo, int vertex ) const
+  ::getCoord ( const ElementInfo &elementInfo, int vertex ) const
   {
     assert( (vertex >= 0) && (vertex <= dim) );
 #if DUNE_ALBERTA_CACHE_COORDINATES
@@ -478,14 +480,14 @@ namespace Dune
 #else
     return elementInfo.coordinate( vertex );
 #endif
-  } 
+  }
 
 
-template < int dim, int dimworld >
-inline int AlbertaGrid < dim, dimworld >::maxLevel() const
-{
-  return maxlevel_;
-}
+  template < int dim, int dimworld >
+  inline int AlbertaGrid < dim, dimworld >::maxLevel() const
+  {
+    return maxlevel_;
+  }
 
 
   template< int dim, int dimworld >
@@ -517,42 +519,42 @@ inline int AlbertaGrid < dim, dimworld >::maxLevel() const
   }
 
 
-template < int dim, int dimworld >
-inline const typename AlbertaGrid < dim, dimworld > :: Traits :: LevelIndexSet & 
-AlbertaGrid < dim, dimworld > :: levelIndexSet (int level) const
-{
-  // assert that given level is in range 
-  assert( (level >= 0) && (level < (int)levelIndexVec_.size()) );
-
-  if( levelIndexVec_[ level ] == 0 )
+  template < int dim, int dimworld >
+  inline const typename AlbertaGrid < dim, dimworld > :: Traits :: LevelIndexSet &
+  AlbertaGrid < dim, dimworld > :: levelIndexSet (int level) const
   {
-    levelIndexVec_[ level ] = new typename GridFamily::LevelIndexSetImp ( dofNumbering_ );
-    levelIndexVec_[ level ]->update( lbegin< 0 >( level ), lend< 0 >( level ) );
-  }
-  return *(levelIndexVec_[ level ]);
-}
+    // assert that given level is in range
+    assert( (level >= 0) && (level < (int)levelIndexVec_.size()) );
 
-template < int dim, int dimworld >
-inline const typename AlbertaGrid < dim, dimworld > :: Traits :: LeafIndexSet & 
-AlbertaGrid < dim, dimworld > :: leafIndexSet () const
-{
-  if( leafIndexSet_ == 0 )
+    if( levelIndexVec_[ level ] == 0 )
+    {
+      levelIndexVec_[ level ] = new typename GridFamily::LevelIndexSetImp ( dofNumbering_ );
+      levelIndexVec_[ level ]->update( lbegin< 0 >( level ), lend< 0 >( level ) );
+    }
+    return *(levelIndexVec_[ level ]);
+  }
+
+  template < int dim, int dimworld >
+  inline const typename AlbertaGrid < dim, dimworld > :: Traits :: LeafIndexSet &
+  AlbertaGrid < dim, dimworld > :: leafIndexSet () const
   {
-    leafIndexSet_ = new typename GridFamily::LeafIndexSetImp( dofNumbering_ );
-    leafIndexSet_->update( leafbegin< 0 >(), leafend< 0 >() );
+    if( leafIndexSet_ == 0 )
+    {
+      leafIndexSet_ = new typename GridFamily::LeafIndexSetImp( dofNumbering_ );
+      leafIndexSet_->update( leafbegin< 0 >(), leafend< 0 >() );
+    }
+    return *leafIndexSet_;
   }
-  return *leafIndexSet_;
-}
 
 
-  template < int dim, int dimworld > 
+  template < int dim, int dimworld >
   inline void AlbertaGrid < dim, dimworld >::calcExtras ()
   {
     // determine new maxlevel
     maxlevel_ = levelProvider_.maxLevel();
     assert( (maxlevel_ >= 0) && (maxlevel_ < MAXL) );
 
-    // unset up2Dat status, if lbegin is called then this status is updated 
+    // unset up2Dat status, if lbegin is called then this status is updated
     for( int l = 0; l < MAXL; ++l )
       levelMarkerVector_[ l ].clear();
 
@@ -560,7 +562,7 @@ AlbertaGrid < dim, dimworld > :: leafIndexSet () const
     leafMarkerVector_.clear();
 
     sizeCache_.reset();
-    
+
     // update index sets (if they exist)
     if( leafIndexSet_ != 0 )
       leafIndexSet_->update( leafbegin< 0 >(), leafend< 0 >() );
@@ -575,22 +577,22 @@ AlbertaGrid < dim, dimworld > :: leafIndexSet () const
   template< int dim, int dimworld >
   template< GrapeIOFileFormatType format >
   inline bool AlbertaGrid< dim, dimworld >
-    ::writeGrid ( const std::string &filename, ctype time ) const
+  ::writeGrid ( const std::string &filename, ctype time ) const
   {
     switch( format )
     {
-      case xdr:
-        return writeGridXdr( filename, time );
+    case xdr :
+      return writeGridXdr( filename, time );
 
-      case ascii:
-        DUNE_THROW( NotImplemented, "AlbertaGrid does not support writeGrid< ascii >." );
+    case ascii :
+      DUNE_THROW( NotImplemented, "AlbertaGrid does not support writeGrid< ascii >." );
 
-        // write leaf grid as macro triangulation 
-        //int ret = ALBERTA write_macro( mesh_ , filename.c_str() );
-        //return (ret == 1) ? true : false;
+    // write leaf grid as macro triangulation
+    //int ret = ALBERTA write_macro( mesh_ , filename.c_str() );
+    //return (ret == 1) ? true : false;
 
-      default:
-        DUNE_THROW( NotImplemented, "AlbertaGrid: Unknown output format: " << format << "." );
+    default :
+      DUNE_THROW( NotImplemented, "AlbertaGrid: Unknown output format: " << format << "." );
     }
     return false;
   }
@@ -599,28 +601,28 @@ AlbertaGrid < dim, dimworld > :: leafIndexSet () const
   template< int dim, int dimworld >
   template< GrapeIOFileFormatType format >
   inline bool AlbertaGrid< dim, dimworld >
-    ::readGrid ( const std::string &filename, ctype &time )
+  ::readGrid ( const std::string &filename, ctype &time )
   {
     switch( format )
     {
-      case xdr:
-        return readGridXdr( filename, time );
+    case xdr :
+      return readGridXdr( filename, time );
 
-      case ascii:
-        DUNE_THROW( NotImplemented, "AlbertaGrid does not support readGrid< ascii >." );
+    case ascii :
+      DUNE_THROW( NotImplemented, "AlbertaGrid does not support readGrid< ascii >." );
 
-        //return readGridAscii (filename , time );
+    //return readGridAscii (filename , time );
 
-      default:
-        DUNE_THROW( NotImplemented, "AlbertaGrid: Unknown output format: " << format << "." );
+    default :
+      DUNE_THROW( NotImplemented, "AlbertaGrid: Unknown output format: " << format << "." );
     }
     return false;
   }
 
 
-  template< int dim, int dimworld > 
+  template< int dim, int dimworld >
   inline bool AlbertaGrid< dim, dimworld >
-    ::writeGridXdr ( const std::string &filename, ctype time ) const
+  ::writeGridXdr ( const std::string &filename, ctype time ) const
   {
     if( filename.size() <= 0 )
       DUNE_THROW( AlbertaIOError, "No filename given to writeGridXdr." );
@@ -628,14 +630,12 @@ AlbertaGrid < dim, dimworld > :: leafIndexSet () const
   }
 
 
-  template< int dim, int dimworld > 
+  template< int dim, int dimworld >
   inline bool AlbertaGrid< dim, dimworld >
-    ::readGridXdr ( const std::string &filename, ctype &time )
+  ::readGridXdr ( const std::string &filename, ctype &time )
   {
-    typedef Alberta::FillFlags< dim > FillFlags;
-
     //removeMesh();
-    
+
     if( filename.size() <= 0 )
       return false;
 
@@ -646,42 +646,13 @@ AlbertaGrid < dim, dimworld > :: leafIndexSet () const
     setup();
     hIndexSet_.read( filename );
 
-    // calc maxlevel and indexOnLevel and so on 
+    // calc maxlevel and indexOnLevel and so on
     calcExtras();
 
     return true;
   }
 
 
-#if 0
-template < int dim, int dimworld > 
-inline bool AlbertaGrid< dim, dimworld >
-  ::readGridAscii ( const std::string &filename, ctype &time )
-{
-  removeMesh(); // delete all objects 
-  
-  mesh_.create( "AlbertaGrid", filename.c_str() );
-  
-  time = 0.0;
-    
-  // unset up2Dat status, if lbegin is called then this status is updated 
-  for( int l = 0; l < MAXL; l++ )
-    levelMarkerVector_[ l ].unsetUp2Date();
-
-  // unset up2Dat status, if leafbegin is called then this status is updated 
-  leafMarkerVector_.unsetUp2Date();
-
-  setup();
-  hIndexSet_.create( dofNumbering_ );
-
-  calcExtras();
-
-  return true;
-}
-#endif
-
-
-
   // AlbertaGrid::AdaptationCallback
   // -------------------------------
 
