@@ -1,6 +1,7 @@
-// -*- tab-width: 4; indent-tabs-mode: nil -*-
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
 #ifdef HAVE_CONFIG_H
-#include "config.h"     
+#include "config.h"
 #endif
 
 #include <cmath>
@@ -76,6 +77,7 @@ public:
     vid[0] = 0; vid[1] = 4; vid[2] = 5; vid[3] = 7; gf.insertElement(type, vid);
     vid[0] = 0; vid[1] = 1; vid[2] = 5; vid[3] = 7; gf.insertElement(type, vid);
 
+    gf.markLongestEdge();
     return Dune::shared_ptr<Grid>(gf.createGrid());
   }
 };
@@ -114,6 +116,7 @@ public:
     // tet at vertex 6
     vid[0] = 2; vid[1] = 4; vid[2] = 6; vid[3] = 7; gf.insertElement(type, vid);
 
+    gf.markLongestEdge();
     return Dune::shared_ptr<Grid>(gf.createGrid());
   }
 };
@@ -168,11 +171,11 @@ int main(int argc, char** argv)
 
     return result;
   }
-  catch (Dune::Exception &e){
+  catch (Dune::Exception &e) {
     std::cerr << "Dune reported error: " << e << std::endl;
     return 1;
   }
-  catch (...){
+  catch (...) {
     std::cerr << "Unknown exception thrown!" << std::endl;
     return 1;
   }
