@@ -1,3 +1,5 @@
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
 #include <config.h>
 
 #include <algorithm>
@@ -20,26 +22,26 @@ namespace Dune
     vertices_.push_back( pos );
   }
 
-  
+
   template< class GridImp >
   void ALU2dGridFactory< GridImp >
-    ::insertElement ( const GeometryType &geometry,
-                      const std::vector< unsigned int > &vertices )
+  ::insertElement ( const GeometryType &geometry,
+                    const std::vector< unsigned int > &vertices )
   {
     switch( elementType )
     {
-      case ALU2DSPACE triangle: 
-        if( !geometry.isTriangle() )
-          DUNE_THROW( GridError, "Only triangles can be inserted into "
-                                 "ALUGrid< 2, " << dimensionworld << ", triangle >." );
-        break;
-      case ALU2DSPACE quadrilateral: 
-        if( !geometry.isCube() )
-          DUNE_THROW( GridError, "Only cubes can be inserted into "
-                                 "ALUGrid< 2, " << dimensionworld << ", quadrilateral >." );
-        break;
-      default:
-        assert( geometry.isSimplex() || geometry.isCube() );
+    case ALU2DSPACE triangle :
+      if( !geometry.isTriangle() )
+        DUNE_THROW( GridError, "Only triangles can be inserted into "
+                    "ALUGrid< 2, " << dimensionworld << ", triangle >." );
+      break;
+    case ALU2DSPACE quadrilateral :
+      if( !geometry.isCube() )
+        DUNE_THROW( GridError, "Only cubes can be inserted into "
+                    "ALUGrid< 2, " << dimensionworld << ", quadrilateral >." );
+      break;
+    default :
+      assert( geometry.isSimplex() || geometry.isCube() );
     }
     if( (geometry.isSimplex() && (vertices.size() != 3))
         || (geometry.isCube() && (vertices.size() != 4)) )
@@ -51,16 +53,16 @@ namespace Dune
 
   template< class GridImp >
   void ALU2dGridFactory< GridImp >
-    ::insertBoundary ( const GeometryType &geometry,
-                       const std::vector< unsigned int > &vertices,
-                       const int id )
+  ::insertBoundary ( const GeometryType &geometry,
+                     const std::vector< unsigned int > &vertices,
+                     const int id )
   {
-    // lines can be either cube or simplex 
+    // lines can be either cube or simplex
     assert( geometry.isSimplex() || geometry.isCube() );
     if( geometry.dim() != dimension-1 )
     {
       DUNE_THROW( GridError, "Only 1-dimensional boundaries can be inserted "
-                             "into a 2-dimensional ALUGrid." );
+                  "into a 2-dimensional ALUGrid." );
     }
     if( vertices.size() != numFaceCorners )
       DUNE_THROW( GridError, "Wrong number of vertices." );
@@ -78,7 +80,7 @@ namespace Dune
 
   template< class GridImp >
   void ALU2dGridFactory< GridImp >
-    ::insertBoundary ( const int element, const int face, const int id )
+  ::insertBoundary ( const int element, const int face, const int id )
   {
     if( (element < 0) || (element >= (int)elements_.size()) )
       DUNE_THROW( RangeError, "ALU2dGridFactory::insertBoundary: invalid element index given." );
@@ -92,20 +94,20 @@ namespace Dune
 
   template< class GridImp >
   void ALU2dGridFactory< GridImp >
-    ::insertBoundaryProjection ( const DuneBoundaryProjectionType &bndProjection )
+  ::insertBoundaryProjection ( const DuneBoundaryProjectionType &bndProjection )
   {
-    if( globalProjection_ ) 
+    if( globalProjection_ )
       DUNE_THROW(InvalidStateException,"You can only insert one globalProjection");
-    
+
     globalProjection_ = &bndProjection;
   }
 
 
   template< class GridImp >
   void ALU2dGridFactory< GridImp >
-    ::insertBoundaryProjection ( const GeometryType &type,
-                                 const std::vector< unsigned int > &vertices,
-                                 const DuneBoundaryProjectionType *projection )
+  ::insertBoundaryProjection ( const GeometryType &type,
+                               const std::vector< unsigned int > &vertices,
+                               const DuneBoundaryProjectionType *projection )
   {
     assert( type.isSimplex() || type.isCube() );
     if( (int)type.dim() != dimension-1 )
@@ -125,7 +127,7 @@ namespace Dune
 
   template< class GridImp >
   void ALU2dGridFactory< GridImp >
-    ::insertBoundarySegment ( const std::vector< unsigned int >& vertices )
+  ::insertBoundarySegment ( const std::vector< unsigned int >& vertices )
   {
     FaceType faceId;
     copyAndSort( vertices, faceId );
@@ -148,8 +150,8 @@ namespace Dune
 
   template< class GridImp >
   void ALU2dGridFactory< GridImp >
-    ::insertBoundarySegment ( const std::vector< unsigned int > &vertices,
-                              const shared_ptr< BoundarySegment< 2, dimensionworld > > &boundarySegment )
+  ::insertBoundarySegment ( const std::vector< unsigned int > &vertices,
+                            const shared_ptr< BoundarySegment< 2, dimensionworld > > &boundarySegment )
   {
     FaceType faceId;
     copyAndSort( vertices, faceId );
@@ -164,16 +166,16 @@ namespace Dune
     GeometryType type;
     if( numVx == 3 )
       type.makeSimplex( dimension-1 );
-    else 
+    else
       type.makeCube( dimension-1 );
 
     std::vector< VertexType > coords( numVx );
     for( size_t i = 0; i < numVx; ++i )
     {
-      // if this assertions is thrown vertices were not inserted at first 
+      // if this assertions is thrown vertices were not inserted at first
       assert( vertices_.size() > vertices[ i ] );
 
-      // get global coordinate and copy it 
+      // get global coordinate and copy it
       const VertexType &x = vertices_[ vertices[ i ] ];
       for( int j = 0; j < dimensionworld; ++j )
         coords[ i ][ j ] = x[ j ];
@@ -183,8 +185,8 @@ namespace Dune
       = new BoundarySegmentWrapperType( type, coords, boundarySegment );
     boundaryProjections_[ faceId ] = prj;
 
-#ifndef NDEBUG 
-    // consistency check 
+#ifndef NDEBUG
+    // consistency check
     for( size_t i = 0; i < numVx; ++i )
     {
       VertexType global = (*prj)( coords [ i ] );
@@ -205,7 +207,7 @@ namespace Dune
 
   template< class GridImp >
   void ALU2dGridFactory< GridImp >
-    :: insertFaceTransformation ( const WorldMatrix &matrix, const WorldVector &shift )
+  :: insertFaceTransformation ( const WorldMatrix &matrix, const WorldVector &shift )
   {
     faceTransformations_.push_back( Transformation( matrix, shift ) );
   }
@@ -213,19 +215,19 @@ namespace Dune
 
   template< class GridImp >
   GridImp  *ALU2dGridFactory< GridImp >
-    ::createGrid ( const bool addMissingBoundaries, bool temporary, const std::string filename )
+  ::createGrid ( const bool addMissingBoundaries, bool temporary, const std::string filename )
   {
     numFacesInserted_ = boundaryIds_.size();
     if( addMissingBoundaries || !faceTransformations_.empty() )
       recreateBoundaryIds();
 
     std::ofstream outfile;
-    std::stringstream temp; 
+    std::stringstream temp;
 
     std::ostream* outptr = 0;
-    if( temporary ) 
+    if( temporary )
       outptr = & temp;
-    else 
+    else
     {
       outfile.open( filename.c_str() , std::ios::out );
       outptr = & outfile;
@@ -234,10 +236,10 @@ namespace Dune
 
     out.setf( std::ios_base::scientific, std::ios_base::floatfield );
     out.precision( 16 );
-    out << "!Triangles" << std::endl; 
+    out << "!Triangles" << std::endl;
 
     const unsigned int numVertices = vertices_.size();
-    // now start writing grid 
+    // now start writing grid
     out << numVertices << std :: endl;
     typedef typename std :: vector< VertexType > :: iterator VertexIteratorType;
     const VertexIteratorType endV = vertices_.end();
@@ -271,9 +273,9 @@ namespace Dune
     const size_t bndProjectionSize = boundaryProjections_.size();
     if( bndProjectionSize > 0 )
     {
-      // the memory is freed by the grid on destruction 
+      // the memory is freed by the grid on destruction
       bndProjections = new BoundaryProjectionVector( boundarySegments,
-                                                    (DuneBoundaryProjectionType*)0 );
+                                                     (DuneBoundaryProjectionType*)0 );
     }
 
     const BoundaryIdIterator endB = boundaryIds_.end();
@@ -299,22 +301,22 @@ namespace Dune
 
       if( bndProjectionSize > 0 )
       {
-        // generate boundary segment pointer 
+        // generate boundary segment pointer
         FaceType faceId (boundaryId.first);
         std::sort( faceId.begin(), faceId.end() );
 
         const DuneBoundaryProjectionType* projection = boundaryProjections_[ faceId ];
 
         // if no projection given we use global projection, otherwise identity
-        if( ! projection && globalProjection_ ) 
+        if( ! projection && globalProjection_ )
         {
           typedef BoundaryProjectionWrapper< dimensionworld > ProjectionWrapperType;
-          // we need to wrap the global projection because of 
-          // deleting in desctructor of ALUGrid 
+          // we need to wrap the global projection because of
+          // deleting in desctructor of ALUGrid
           projection = new ProjectionWrapperType( *globalProjection_ );
         }
 
-        // copy pointer 
+        // copy pointer
         (*bndProjections)[ segmentIndex ] = projection;
       }
     }
@@ -328,16 +330,16 @@ namespace Dune
 
     std::istream& inFile = temp;
 
-    // if we have a vector reset global projection 
-    // because empty positions are filled with global projection anyway  
+    // if we have a vector reset global projection
+    // because empty positions are filled with global projection anyway
     if( bndProjections ) globalProjection_ = 0;
 
-    // ALUGrid is taking ownership of the bndProjections pointer 
+    // ALUGrid is taking ownership of the bndProjections pointer
     Grid *grid = createGridObj( temporary, filename, inFile, bndProjections );
 
-    // remove pointers 
+    // remove pointers
     globalProjection_ = 0;
-    // is removed by the grid 
+    // is removed by the grid
     bndProjections    = 0;
 
     return grid;
@@ -345,36 +347,36 @@ namespace Dune
 
   template< class GridImp >
   inline void ALU2dGridFactory< GridImp >
-    ::generateFace ( const ElementType &element, const int f, FaceType &face )
+  ::generateFace ( const ElementType &element, const int f, FaceType &face )
   {
     if (element.size() == 3)
       switch (f)
       {
-        case 0:
-          face[0]=element[0]; face[1]=element[1];
-          break;
-        case 1:
-          face[0]=element[0]; face[1]=element[2];
-          break;
-        case 2:
-          face[0]=element[1]; face[1]=element[2];
-          break;
+      case 0 :
+        face[0]=element[0]; face[1]=element[1];
+        break;
+      case 1 :
+        face[0]=element[0]; face[1]=element[2];
+        break;
+      case 2 :
+        face[0]=element[1]; face[1]=element[2];
+        break;
       }
     else
       switch (f)
       {
-        case 0:
-          face[0]=element[0]; face[1]=element[2];
-          break;
-        case 1:
-          face[0]=element[1]; face[1]=element[3];
-          break;
-        case 2:
-          face[0]=element[0]; face[1]=element[1];
-          break;
-        case 3:
-          face[0]=element[2]; face[1]=element[3];
-          break;
+      case 0 :
+        face[0]=element[0]; face[1]=element[2];
+        break;
+      case 1 :
+        face[0]=element[1]; face[1]=element[3];
+        break;
+      case 2 :
+        face[0]=element[0]; face[1]=element[1];
+        break;
+      case 3 :
+        face[0]=element[2]; face[1]=element[3];
+        break;
       }
   }
 
@@ -382,7 +384,7 @@ namespace Dune
   template< class GridImp >
   typename ALU2dGridFactory< GridImp >::FaceMap::const_iterator
   ALU2dGridFactory< GridImp >
-    ::findPeriodicNeighbor ( const FaceMap &faceMap, const FaceType &key ) const
+  ::findPeriodicNeighbor ( const FaceMap &faceMap, const FaceType &key ) const
   {
     typedef typename FaceTransformationVector::const_iterator TrafoIterator;
     typedef typename FaceMap::const_iterator FaceMapIterator;
@@ -423,7 +425,7 @@ namespace Dune
 
   template< class GridImp >
   void ALU2dGridFactory< GridImp >
-    ::reinsertBoundary ( const FaceMap &faceMap, const typename FaceMap::const_iterator &pos, const int id )
+  ::reinsertBoundary ( const FaceMap &faceMap, const typename FaceMap::const_iterator &pos, const int id )
   {
     typedef typename PeriodicNeighborMap::const_iterator PeriodicNbIterator;
     typedef typename FaceMap::const_iterator FaceMapIterator;
@@ -462,7 +464,6 @@ namespace Dune
   void ALU2dGridFactory< GridImp >::recreateBoundaryIds ( const int defaultId )
   {
     typedef typename FaceMap::iterator FaceIterator;
-    typedef typename PeriodicNeighborMap::const_iterator PeriodicNbIterator;
 
     if( defaultId <= 0 )
       DUNE_THROW( GridError, "Boundary ids must be positive." );
@@ -477,7 +478,7 @@ namespace Dune
         FaceType key;
         generateFace( elements_[ n ], face, key );
         std::sort( key.begin(), key.end() );
-       
+
         const FaceIterator pos = faceMap.find( key );
         if( pos != faceMap.end() )
           faceMap.erase( key );
@@ -490,7 +491,7 @@ namespace Dune
     BoundaryIdVector boundaryIds;
     boundaryIds_.swap( boundaryIds );
     assert( boundaryIds_.size() == 0 );
-    
+
     // add all current boundary ids again (with their reordered keys)
     typedef typename BoundaryIdVector::iterator BoundaryIterator;
     const BoundaryIterator bndEnd = boundaryIds.end();
@@ -537,7 +538,7 @@ template class Dune::ALU2dGridFactory< Dune::ALUConformGrid< 2, 3 > >;
 template class Dune::ALU2dGridFactory< Dune::ALUSimplexGrid< 2, 3 > >;
 template class Dune::ALU2dGridFactory< Dune::ALUCubeGrid< 2, 3 > >;
 
-// new versions 
+// new versions
 template class Dune::ALU2dGridFactory< Dune::ALUGrid< 2, 3, Dune::simplex, Dune::conforming,    Dune::No_Comm > >;
 template class Dune::ALU2dGridFactory< Dune::ALUGrid< 2, 3, Dune::simplex, Dune::nonconforming, Dune::No_Comm > >;
 template class Dune::ALU2dGridFactory< Dune::ALUGrid< 2, 3, Dune::cube,    Dune::nonconforming, Dune::No_Comm > >;
