@@ -1,5 +1,7 @@
-#ifndef DUNE_ALU3DGRID_FACTORY_CC 
-#define DUNE_ALU3DGRID_FACTORY_CC 
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
+#ifndef DUNE_ALU3DGRID_FACTORY_CC
+#define DUNE_ALU3DGRID_FACTORY_CC
 
 #if COMPILE_ALUGRID_INLINE == 0
 #include <config.h>
@@ -17,20 +19,20 @@
 #if HAVE_ALUGRID
 
 #if COMPILE_ALUGRID_INLINE
-#define alu_inline inline 
+#define alu_inline inline
 #else
 #define alu_inline
 #endif
 
 namespace Dune
 {
-  
+
   template< class ALUGrid >
   alu_inline
   ALU3dGridFactory< ALUGrid > :: ~ALU3dGridFactory ()
   {}
 
-  
+
   template< class ALUGrid >
   alu_inline
   void ALU3dGridFactory< ALUGrid > :: insertVertex ( const VertexType &pos )
@@ -39,7 +41,7 @@ namespace Dune
       DUNE_THROW( GridError, "ALU3dGridFactory allows insertion only for rank 0." );
 
     vertices_.push_back( std::make_pair( pos, vertices_.size() ) );
- }
+  }
 
 
   template< class ALUGrid >
@@ -52,17 +54,17 @@ namespace Dune
     return vertexId;
   }
 
-  
+
   template< class ALUGrid >
   alu_inline
   void ALU3dGridFactory< ALUGrid >
-    :: insertElement ( const GeometryType &geometry,
-                       const std::vector< VertexId > &vertices )
+  :: insertElement ( const GeometryType &geometry,
+                     const std::vector< VertexId > &vertices )
   {
     assertGeometryType( geometry );
     if( geometry.dim() != dimension )
       DUNE_THROW( GridError, "Only 3-dimensional elements can be inserted "
-                             "into a 3-dimensional ALUGrid." );
+                  "into a 3-dimensional ALUGrid." );
     if( vertices.size() != numCorners )
       DUNE_THROW( GridError, "Wrong number of vertices." );
 
@@ -73,15 +75,15 @@ namespace Dune
   template< class ALUGrid >
   alu_inline
   void ALU3dGridFactory< ALUGrid >
-    :: insertBoundary ( const GeometryType &geometry,
-                        const std::vector< VertexId > &vertices,
-                        const int id )
+  :: insertBoundary ( const GeometryType &geometry,
+                      const std::vector< VertexId > &vertices,
+                      const int id )
   {
     assertGeometryType( geometry );
     if( geometry.dim() != dimension-1 )
     {
       DUNE_THROW( GridError, "Only 2-dimensional boundaries can be inserted "
-                             "into a 3-dimensional ALUGrid." );
+                  "into a 3-dimensional ALUGrid." );
     }
     if( vertices.size() != numFaceCorners )
       DUNE_THROW( GridError, "Wrong number of vertices." );
@@ -101,7 +103,7 @@ namespace Dune
   template< class ALUGrid >
   alu_inline
   void ALU3dGridFactory< ALUGrid >
-    ::insertBoundary ( const int element, const int face, const int id )
+  ::insertBoundary ( const int element, const int face, const int id )
   {
     if( (element < 0) || (element >= (int)elements_.size()) )
       DUNE_THROW( RangeError, "ALU3dGridFactory::insertBoundary: invalid element index given." );
@@ -114,19 +116,19 @@ namespace Dune
 
   template< class ALUGrid >
   alu_inline
-  void ALU3dGridFactory< ALUGrid > :: 
-  insertBoundaryProjection( const DuneBoundaryProjectionType& bndProjection ) 
+  void ALU3dGridFactory< ALUGrid > ::
+  insertBoundaryProjection( const DuneBoundaryProjectionType& bndProjection )
   {
-    if( globalProjection_ ) 
+    if( globalProjection_ )
       DUNE_THROW(InvalidStateException,"You can only insert one globalProjection");
 
-    globalProjection_ = &bndProjection; 
+    globalProjection_ = &bndProjection;
   }
 
 
   template< class ALUGrid >
   alu_inline
-  void ALU3dGridFactory< ALUGrid > :: 
+  void ALU3dGridFactory< ALUGrid > ::
   insertBoundaryProjection ( const GeometryType &type,
                              const std::vector< VertexId > &vertices,
                              const DuneBoundaryProjectionType *projection )
@@ -150,7 +152,7 @@ namespace Dune
   template< class ALUGrid >
   alu_inline
   void ALU3dGridFactory< ALUGrid >
-    ::insertFaceTransformation ( const WorldMatrix &matrix, const WorldVector &shift )
+  ::insertFaceTransformation ( const WorldMatrix &matrix, const WorldVector &shift )
   {
     faceTransformations_.push_back( Transformation( matrix, shift ) );
   }
@@ -166,7 +168,7 @@ namespace Dune
   template< class ALUGrid >
   alu_inline
   ALUGrid *ALU3dGridFactory< ALUGrid >
-    ::createGrid ( const bool addMissingBoundaries, const std::string dgfName )
+  ::createGrid ( const bool addMissingBoundaries, const std::string dgfName )
   {
     return createGrid( addMissingBoundaries, true, dgfName );
   }
@@ -174,9 +176,9 @@ namespace Dune
   template< class ALUGrid >
   alu_inline
   ALUGrid *ALU3dGridFactory< ALUGrid >
-    ::createGrid ( const bool addMissingBoundaries, bool temporary, const std::string name )
+  ::createGrid ( const bool addMissingBoundaries, bool temporary, const std::string name )
   {
-    typedef typename BoundaryIdMap :: iterator  BoundaryIdIteratorType;
+    typedef typename BoundaryIdMap :: iterator BoundaryIdIteratorType;
     BoundaryProjectionVector* bndProjections = 0;
 
     correctElementOrientation();
@@ -184,26 +186,26 @@ namespace Dune
     if( addMissingBoundaries || ! faceTransformations_.empty() )
       recreateBoundaryIds();
 
-    // if dump file should be written 
+    // if dump file should be written
     if( allowGridGeneration_ && !temporary )
     {
       std::string filename ( name );
-      
+
       std::ofstream out( filename.c_str() );
       out.setf( std::ios_base::scientific, std::ios_base::floatfield );
       out.precision( 16 );
       if( elementType == tetra )
-        out << "!Tetrahedra"; 
+        out << "!Tetrahedra";
       else
         out << "!Hexahedra";
 
       const unsigned int numVertices = vertices_.size();
-      // print information about vertices and elements 
-      // to header to have an easy check 
+      // print information about vertices and elements
+      // to header to have an easy check
       out << "  ( noVertices = " << numVertices;
       out << " | noElements = " << elements_.size() << " )" << std :: endl;
 
-      // now start writing grid 
+      // now start writing grid
       out << numVertices << std :: endl;
       typedef typename VertexVector::iterator VertexIteratorType;
       const VertexIteratorType endV = vertices_.end();
@@ -230,7 +232,7 @@ namespace Dune
 
         out << element[ 0 ];
         for( unsigned int i = 1; i < numCorners; ++i )
-          out << "  " << element[ i ];
+          out << " " << element[ i ];
         out << std :: endl;
       }
 
@@ -239,10 +241,10 @@ namespace Dune
       for( BoundaryIdIteratorType it = boundaryIds_.begin(); it != endB; ++it )
       {
         const std::pair< FaceType, int > &boundaryId = *it;
-        out << (-boundaryId.second) << "  " << numFaceCorners;
+        out << (-boundaryId.second) << " " << numFaceCorners;
 
         for( unsigned int i = 0; i < numFaceCorners; ++i )
-          out << "  " << boundaryId.first[ i ];
+          out << " " << boundaryId.first[ i ];
         out << std::endl;
       }
       const typename PeriodicBoundaryVector::iterator endP = periodicBoundaries_.end();
@@ -250,91 +252,92 @@ namespace Dune
       {
         typedef typename ALU3dBasicImplTraits< MPICommunicatorType >::HBndSegType HBndSegType;
         const std::pair< BndPair, BndPair > &facePair = *it;
-        out << (-HBndSegType::periodic) << "  " << (2*numFaceCorners);
+        out << (-HBndSegType::periodic) << " " << (2*numFaceCorners);
         for( unsigned int i = 0; i < numFaceCorners; ++i )
-          out << "  " << facePair.first.first[ numFaceCorners == 3 ? (3 - i) % 3 : i ];
+          out << " " << facePair.first.first[ numFaceCorners == 3 ? (3 - i) % 3 : i ];
         for( unsigned int i = 0; i < numFaceCorners; ++i )
-          out << "  " << facePair.second.first[ numFaceCorners == 3 ? (3 - i) % 3 : i ];
+          out << " " << facePair.second.first[ numFaceCorners == 3 ? (3 - i) % 3 : i ];
         out << std::endl;
       }
 
       // write global vertex ids
       for( unsigned int i = 0; i < numVertices; ++i )
-        out << globalId( i ) << "  -1" << std :: endl;
+        out << globalId( i ) << " -1" << std :: endl;
       out.close();
     }
 
     const size_t boundarySegments = boundaryIds_.size();
 
     const size_t bndProjectionSize = boundaryProjections_.size();
-    if( bndProjectionSize > 0 ) 
+    if( bndProjectionSize > 0 )
     {
-      // the memory is freed by the grid on destruction 
+      // the memory is freed by the grid on destruction
       bndProjections = new BoundaryProjectionVector( boundarySegments,
-                                                    (DuneBoundaryProjectionType*) 0 );
+                                                     (DuneBoundaryProjectionType*) 0 );
       const BoundaryIdIteratorType endB = boundaryIds_.end();
       int segmentIndex = 0;
       for( BoundaryIdIteratorType it = boundaryIds_.begin(); it != endB; ++it, ++segmentIndex )
       {
-        // generate boundary segment pointer 
+        // generate boundary segment pointer
         FaceType faceId ( (*it).first);
         std::sort( faceId.begin(), faceId.end() );
 
         const DuneBoundaryProjectionType* projection = boundaryProjections_[ faceId ];
 
-        // if no projection given we use global projection, otherwise identity 
+        // if no projection given we use global projection, otherwise identity
         if( ! projection && globalProjection_ )
         {
           typedef BoundaryProjectionWrapper< dimensionworld > ProjectionWrapperType;
-          // we need to wrap the global projection because of 
-          // delete in desctructor of ALUGrid 
+          // we need to wrap the global projection because of
+          // delete in desctructor of ALUGrid
           projection = new ProjectionWrapperType( *globalProjection_ );
           assert( projection );
         }
 
-        // copy pointer 
+        // copy pointer
         (*bndProjections)[ segmentIndex ] = projection;
       }
     } // if( allowGridGeneration_ && !temporary )
 
-    // free memory 
+    // free memory
     boundaryProjections_.clear();
 
-    // if we have a vector reset global projection 
-    // because empty positions are filled with global projection anyway  
+    // if we have a vector reset global projection
+    // because empty positions are filled with global projection anyway
     if( bndProjections ) globalProjection_ = 0;
 
-    // ALUGrid is taking ownership of bndProjections 
-    // and is going to delete this pointer 
+    // ALUGrid is taking ownership of bndProjections
+    // and is going to delete this pointer
     Grid* grid = createGridObj( bndProjections , name );
     assert( grid );
 
-    // remove pointers 
+    // remove pointers
     globalProjection_ = 0;
-    // is removed by grid instance 
+    // is removed by grid instance
     bndProjections    = 0;
 
-    // insert grid using ALUGrid macro grid builder   
+    // insert grid using ALUGrid macro grid builder
     if( !vertices_.empty() )
     {
       ALU3DSPACE MacroGridBuilder mgb ( grid->getBuilder()
 #ifdef ALUGRID_VERTEX_PROJECTION
-          , grid->vertexProjection() 
+                                        , grid->vertexProjection()
 #endif
-          );
+                                        );
 
-      // now start inserting grid 
-      const int vxSize = vertices_.size(); 
+      // now start inserting grid
+      const int vxSize = vertices_.size();
       for( int vxIdx = 0; vxIdx < vxSize ; ++vxIdx )
       {
-        // insert vertex 
+        // insert vertex
         const VertexType &vertex = position( vxIdx );
         mgb.InsertUniqueVertex( vertex[ 0 ], vertex[ 1 ], vertex[ 2 ], globalId( vxIdx ) );
       }
 
       typedef typename ElementVector::iterator ElementIteratorType;
       const ElementIteratorType endE = elements_.end();
-      for( ElementIteratorType it = elements_.begin(); it != endE; ++it )
+      unsigned int elemIndex = 0;
+      for( ElementIteratorType it = elements_.begin(); it != endE; ++it, ++elemIndex )
       {
         if( elementType == hexa )
         {
@@ -354,9 +357,13 @@ namespace Dune
             const unsigned int j = ElementTopologyMappingType::dune2aluVertex( i );
             element[ j ] = globalId( (*it)[ i ] );
           }
-          mgb.InsertUniqueTetra( element );
+          mgb.InsertUniqueTetra( element
+#ifdef ALUGRID_3D_CONFORMING_REFINEMENT
+                                 , (elemIndex % 2)
+#endif
+                                 );
         }
-        else 
+        else
           DUNE_THROW( GridError, "Invalid element type");
       }
 
@@ -384,7 +391,7 @@ namespace Dune
           }
           mgb.InsertUniqueHbnd3( bndface, bndType );
         }
-        else 
+        else
           DUNE_THROW( GridError, "Invalid element type");
       }
       const typename PeriodicBoundaryVector::iterator endP = periodicBoundaries_.end();
@@ -402,10 +409,10 @@ namespace Dune
 
 #ifdef ALUGRID_PERIODIC_BOUNDARY_PARALLEL
           typedef typename ALU3DSPACE Gitter::hbndseg::bnd_t bnd_t ;
-          bnd_t bndId[ 2 ] = { bnd_t( facePair.first.second ), 
+          bnd_t bndId[ 2 ] = { bnd_t( facePair.first.second ),
                                bnd_t( facePair.second.second ) };
           mgb.InsertUniquePeriodic4( perel, bndId );
-#else 
+#else
           mgb.InsertUniquePeriodic4( perel );
 #endif
 
@@ -420,23 +427,23 @@ namespace Dune
           }
 #ifdef ALUGRID_PERIODIC_BOUNDARY_PARALLEL
           typedef typename ALU3DSPACE Gitter::hbndseg::bnd_t bnd_t ;
-          bnd_t bndId[ 2 ] = { bnd_t( facePair.first.second ), 
+          bnd_t bndId[ 2 ] = { bnd_t( facePair.first.second ),
                                bnd_t( facePair.second.second ) };
           mgb.InsertUniquePeriodic3( perel, bndId );
-#else 
+#else
           mgb.InsertUniquePeriodic3( perel );
 #endif
         }
-        else 
+        else
           DUNE_THROW( GridError, "Invalid element type" );
       }
     }
 
-    // clear vertices  
+    // clear vertices
     vertices_ = VertexVector();
-    // clear elements 
+    // clear elements
     elements_ = ElementVector();
-    // free memory 
+    // free memory
     boundaryIds_.clear();
 
 #ifdef ALUGRID_EXPORT_MACROGRID_CHANGES
@@ -445,14 +452,14 @@ namespace Dune
       // make changes in macro grid known in every partition
       grid->duneNotifyMacroGridChanges();
     }
-#else 
+#else
     if( grid->comm().size() > 1 )
       DUNE_THROW(NotImplemented,"ALUGrid factory not working in parallel right now!");
 #endif // #ifdef ALUGRID_EXPORT_MACROGRID_CHANGES
 
-    // reset wasRefined flags 
+    // reset wasRefined flags
     grid->postAdapt();
-    // update additional information on grid 
+    // update additional information on grid
     grid->calcExtras();
 
     return grid;
@@ -462,7 +469,7 @@ namespace Dune
   template< class ALUGrid >
   alu_inline
   void ALU3dGridFactory< ALUGrid >
-    ::generateFace ( const ElementType &element, const int f, FaceType &face )
+  ::generateFace ( const ElementType &element, const int f, FaceType &face )
   {
     typedef ElementTopologyMapping< elementType > ElementTopologyMappingType;
     const int falu = ElementTopologyMappingType :: generic2aluFace( f );
@@ -525,9 +532,9 @@ namespace Dune
   template< class ALUGrid >
   alu_inline
   bool ALU3dGridFactory< ALUGrid >
-    ::identifyFaces ( const Transformation &transformation, 
-                      const FaceType &key1, const FaceType &key2,
-                      const int defaultId )
+  ::identifyFaces ( const Transformation &transformation,
+                    const FaceType &key1, const FaceType &key2,
+                    const int defaultId )
   {
     WorldVector w = transformation.evaluate( position( key1[ 0 ] ) );
     int org = -1;
@@ -553,12 +560,12 @@ namespace Dune
     int bndId[ 2 ] = { 20, 20 };
     FaceType keys[ 2 ] = { key1, key2 };
 
-    for( int i=0; i<2; ++i ) 
+    for( int i=0; i<2; ++i )
     {
       typedef typename BoundaryIdMap :: iterator iterator ;
       iterator pos = boundaryIds_.find( keys[ i ] );
-    
-      if( pos != boundaryIds_.end() ) 
+
+      if( pos != boundaryIds_.end() )
       {
         bndId[ i ] = (*pos).second ;
         boundaryIds_.erase( pos );
@@ -577,8 +584,8 @@ namespace Dune
   template< class ALUGrid >
   alu_inline
   void ALU3dGridFactory< ALUGrid >
-    ::searchPeriodicNeighbor ( FaceMap &faceMap, const typename FaceMap::iterator &pos,
-                               const int defaultId )
+  ::searchPeriodicNeighbor ( FaceMap &faceMap, const typename FaceMap::iterator &pos,
+                             const int defaultId )
   {
     typedef typename FaceTransformationVector::const_iterator TrafoIterator;
     typedef typename FaceMap::iterator FaceMapIterator;
@@ -597,7 +604,7 @@ namespace Dune
         const TrafoIterator trend = faceTransformations_.end();
         for( TrafoIterator trit = faceTransformations_.begin(); trit != trend; ++trit )
         {
-          if( identifyFaces( *trit, key1, key2, defaultId) || 
+          if( identifyFaces( *trit, key1, key2, defaultId) ||
               identifyFaces( *trit, key2, key1, defaultId) )
           {
             faceMap.erase( fit );
@@ -613,7 +620,7 @@ namespace Dune
   template< class ALUGrid >
   alu_inline
   void ALU3dGridFactory< ALUGrid >
-    ::reinsertBoundary ( const FaceMap &faceMap, const typename FaceMap::const_iterator &pos, const int id )
+  ::reinsertBoundary ( const FaceMap &faceMap, const typename FaceMap::const_iterator &pos, const int id )
   {
     insertBoundary( pos->second.first, pos->second.second, id );
   }
@@ -622,7 +629,7 @@ namespace Dune
   template< class ALUGrid >
   alu_inline
   void ALU3dGridFactory< ALUGrid >
-    ::recreateBoundaryIds ( const int defaultId )
+  ::recreateBoundaryIds ( const int defaultId )
   {
     typedef typename FaceMap::iterator FaceIterator;
     FaceMap faceMap;
@@ -635,7 +642,7 @@ namespace Dune
         FaceType key;
         generateFace( elements_[ n ], face, key );
         std::sort( key.begin(), key.end() );
-       
+
         const FaceIterator pos = faceMap.find( key );
         if( pos != faceMap.end() )
           faceMap.erase( key );
@@ -685,7 +692,7 @@ namespace Dune
   template class ALU3dGridFactory< ALUGrid< 3, 3, simplex, conforming > >;
 #endif
 }
-#endif // end ENABLE_ALUGRID 
+#endif // end ENABLE_ALUGRID
 #undef alu_inline
 
 #endif // end DUNE_ALU3DGRID_FACTORY_CC
