@@ -1,3 +1,5 @@
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
 /**\cond */
 namespace Dune
 {
@@ -6,20 +8,20 @@ namespace Dune
   {
 
     struct ALU2dGridParameterBlock
-    : public GridParameterBlock
+      : public GridParameterBlock
     {
       ALU2dGridParameterBlock( std::istream &in, const bool verbose )
-      : GridParameterBlock( in ),
-        tolerance_( 1e-8 )
+        : GridParameterBlock( in ),
+          tolerance_( 1e-8 )
       {
         if( findtoken( "tolerance" ) )
         {
           double x;
           if( getnextentry(x) )
             tolerance_ = x;
-          else 
+          else
           {
-            if( verbose ) 
+            if( verbose )
             {
               dwarn << "GridParameterBlock: found keyword `tolerance' but no value, "
                     << "defaulting to `" <<  tolerance_ <<"'!" << std::endl;
@@ -46,18 +48,18 @@ namespace Dune
 
   }
 
-  template < class G > 
-  inline bool DGFBaseFactory< G > :: 
-  generateALUGrid( const ALUGridElementType eltype, 
-                   std::istream &file, MPICommunicatorType communicator, 
+  template < class G >
+  inline bool DGFBaseFactory< G > ::
+  generateALUGrid( const ALUGridElementType eltype,
+                   std::istream &file, MPICommunicatorType communicator,
                    const std::string &filename )
   {
     typedef G DGFGridType ;
 
-    const int dimworld = DGFGridType :: dimensionworld ; 
-    dgf_.element = ( eltype == simplex) ? 
-                        DuneGridFormatParser::Simplex : 
-                        DuneGridFormatParser::Cube ;
+    const int dimworld = DGFGridType :: dimensionworld ;
+    dgf_.element = ( eltype == simplex) ?
+                   DuneGridFormatParser::Simplex :
+                   DuneGridFormatParser::Cube ;
     dgf_.dimgrid = dimworld;
     dgf_.dimw = dimworld;
 
@@ -80,7 +82,7 @@ namespace Dune
       if( !dgf_.readDuneGrid( file, dimworld, dimworld ) )
         DUNE_THROW( InvalidStateException, "DGF file not recognized on second call." );
 
-      if( eltype == simplex ) 
+      if( eltype == simplex )
       {
         dgf_.setOrientation( 2, 3 );
       }
@@ -93,9 +95,9 @@ namespace Dune
         factory_.insertVertex( pos );
       }
 
-      GeometryType elementType( (eltype == simplex) ? 
-                                    GeometryType::simplex : 
-                                    GeometryType::cube, dimworld );
+      GeometryType elementType( (eltype == simplex) ?
+                                GeometryType::simplex :
+                                GeometryType::cube, dimworld );
 
       const int nFaces = (eltype == simplex) ? dimworld+1 : 2*dimworld;
       for( int n = 0; n < dgf_.nofelements; ++n )
@@ -123,10 +125,10 @@ namespace Dune
       const size_t numBoundaryProjections = projectionBlock.numBoundaryProjections();
       for( size_t i = 0; i < numBoundaryProjections; ++i )
       {
-        GeometryType type( (eltype == simplex) ? 
-                               GeometryType::simplex : 
-                               GeometryType::cube, 
-                            dimworld-1);
+        GeometryType type( (eltype == simplex) ?
+                           GeometryType::simplex :
+                           GeometryType::cube,
+                           dimworld-1);
 
         const std::vector< unsigned int > &vertices = projectionBlock.boundaryFace( i );
         const DuneBoundaryProjection< dimworld > *projection
@@ -156,49 +158,50 @@ namespace Dune
 
     if ( ! parameter.dumpFileName().empty() )
       grid_ = factory_.createGrid( dgf_.facemap.empty(), false, parameter.dumpFileName() );
-    else 
+    else
       grid_ = factory_.createGrid( dgf_.facemap.empty(), true, filename );
     return true;
   }
 
 
-  inline bool DGFGridFactory< ALUSimplexGrid< 3, 3 > >
-    ::generate( std::istream &file, MPICommunicatorType communicator, const std::string &filename )
+  template < int dimw >
+  inline bool DGFGridFactory< ALUSimplexGrid< 3, dimw > >
+  ::generate( std::istream &file, MPICommunicatorType communicator, const std::string &filename )
   {
-    return DGFBaseFactory< ALUSimplexGrid< 3, 3 > > :: 
-      generateALUGrid( simplex, file, communicator, filename );
+    return DGFBaseFactory< ALUSimplexGrid< 3, dimw > > ::
+           generateALUGrid( simplex, file, communicator, filename );
   }
 
-
-  inline bool DGFGridFactory< ALUCubeGrid< 3, 3 > >
-    ::generate( std::istream &file, MPICommunicatorType communicator, const std::string &filename )
+  template < int dimw >
+  inline bool DGFGridFactory< ALUCubeGrid< 3, dimw > >
+  ::generate( std::istream &file, MPICommunicatorType communicator, const std::string &filename )
   {
-    return DGFBaseFactory< ALUCubeGrid< 3, 3 > > :: 
-      generateALUGrid( cube, file, communicator, filename );
+    return DGFBaseFactory< ALUCubeGrid< 3, dimw > > ::
+           generateALUGrid( cube, file, communicator, filename );
   }
 
   template <ALUGridElementType eltype, ALUGridRefinementType refinementtype, class Comm>
   inline bool DGFGridFactory< ALUGrid< 3, 3, eltype, refinementtype, Comm > >
-    ::generate( std::istream &file, MPICommunicatorType communicator, const std::string &filename )
+  ::generate( std::istream &file, MPICommunicatorType communicator, const std::string &filename )
   {
     return BaseType :: generateALUGrid( eltype, file, communicator, filename );
   }
 
 
-  // ALUGrid 2d version 
+  // ALUGrid 2d version
   //-------------------
 
-  template < class G > 
-  inline bool DGFBaseFactory< G > :: 
-    generateALU2dGrid( const ALUGridElementType eltype, 
-                       std::istream &file, 
-                       MPICommunicatorType communicator, 
-                       const std::string &filename )
+  template < class G >
+  inline bool DGFBaseFactory< G > ::
+  generateALU2dGrid( const ALUGridElementType eltype,
+                     std::istream &file,
+                     MPICommunicatorType communicator,
+                     const std::string &filename )
   {
     const int dimgrid = 2;
-    const int dimworld = G :: dimensionworld ; 
-    dgf_.element = (eltype == simplex) ? 
-        DuneGridFormatParser::Simplex : DuneGridFormatParser::Cube ;
+    const int dimworld = G :: dimensionworld ;
+    dgf_.element = (eltype == simplex) ?
+                   DuneGridFormatParser::Simplex : DuneGridFormatParser::Cube ;
     dgf_.dimgrid = dimgrid;
     dgf_.dimw = dimworld;
 
@@ -206,19 +209,19 @@ namespace Dune
     file.seekg( 0 );
     if( !isDGF )
       return false;
-   
+
     int rank = 0;
 #if HAVE_MPI
     MPI_Comm_rank( communicator, &rank );
 #endif
 
-    // set verbosity of factory only for rank = 0 
+    // set verbosity of factory only for rank = 0
     factory_.setVerbosity( (rank == 0) );
 
-    // only print warnings of ALU2dGridParameterBlock on rank = 0 
+    // only print warnings of ALU2dGridParameterBlock on rank = 0
     dgf::ALU2dGridParameterBlock parameter( file, (rank == 0) );
 
-#if ALU2DGRID_PARALLEL 
+#if ALU2DGRID_PARALLEL
     // for a parallel ALUGrid implementation do the following only on rank 0
     if( rank == 0 )
 #endif
@@ -236,9 +239,9 @@ namespace Dune
         factory_.insertVertex( pos );
       }
 
-      GeometryType elementType( (eltype == simplex) ? 
-                                 GeometryType::simplex : 
-                                 GeometryType::cube, dimgrid );
+      GeometryType elementType( (eltype == simplex) ?
+                                GeometryType::simplex :
+                                GeometryType::cube, dimgrid );
 
       const int nFaces = (eltype == simplex) ? dimgrid+1 : 2*dimgrid;
       for( int n = 0; n < dgf_.nofelements; ++n )
@@ -264,8 +267,8 @@ namespace Dune
       const size_t numBoundaryProjections = projectionBlock.numBoundaryProjections();
       for( size_t i = 0; i < numBoundaryProjections; ++i )
       {
-        GeometryType type( (eltype == simplex) ? 
-                           GeometryType::simplex : 
+        GeometryType type( (eltype == simplex) ?
+                           GeometryType::simplex :
                            GeometryType::cube, dimgrid-1 );
         const std::vector< unsigned int > &vertices = projectionBlock.boundaryFace( i );
         const DuneBoundaryProjection< dimworld > *projection
@@ -295,41 +298,41 @@ namespace Dune
 
     if ( ! parameter.dumpFileName().empty() && (rank == 0) )
       grid_ = factory_.createGrid( dgf_.facemap.empty(), false, parameter.dumpFileName() );
-    else 
+    else
       grid_ = factory_.createGrid( dgf_.facemap.empty(), true, filename );
     return true;
   }
 
-  template <int dimw, ALUGridElementType eltype, 
-            ALUGridRefinementType refinementtype, class Comm >
+  template <int dimw, ALUGridElementType eltype,
+      ALUGridRefinementType refinementtype, class Comm >
   inline bool DGFGridFactory< ALUGrid< 2, dimw, eltype, refinementtype, Comm > >
-    ::generate( std::istream &file, MPICommunicatorType communicator, const std::string &filename )
+  ::generate( std::istream &file, MPICommunicatorType communicator, const std::string &filename )
   {
     return BaseType :: generateALU2dGrid( eltype, file, communicator, filename );
   }
 
   template <int dimw>
   inline bool DGFGridFactory< ALUSimplexGrid< 2, dimw > >
-    ::generate( std::istream &file, MPICommunicatorType communicator, const std::string &filename )
+  ::generate( std::istream &file, MPICommunicatorType communicator, const std::string &filename )
   {
-    return DGFBaseFactory< ALUSimplexGrid< 2, dimw > > :: 
-      generateALU2dGrid( simplex, file, communicator, filename );
+    return DGFBaseFactory< ALUSimplexGrid< 2, dimw > > ::
+           generateALU2dGrid( simplex, file, communicator, filename );
   }
 
   template <int dimw>
   inline bool DGFGridFactory< ALUCubeGrid< 2, dimw > >
-    ::generate( std::istream &file, MPICommunicatorType communicator, const std::string &filename )
+  ::generate( std::istream &file, MPICommunicatorType communicator, const std::string &filename )
   {
-    return DGFBaseFactory< ALUCubeGrid< 2, dimw > > :: 
-      generateALU2dGrid( cube, file, communicator, filename );
+    return DGFBaseFactory< ALUCubeGrid< 2, dimw > > ::
+           generateALU2dGrid( cube, file, communicator, filename );
   }
 
   template <int dimw>
   inline bool DGFGridFactory< ALUConformGrid< 2, dimw > >
-    ::generate( std::istream &file, MPICommunicatorType communicator, const std::string &filename )
+  ::generate( std::istream &file, MPICommunicatorType communicator, const std::string &filename )
   {
-    return DGFBaseFactory< ALUConformGrid< 2, dimw > > :: 
-      generateALU2dGrid( simplex, file, communicator, filename );
+    return DGFBaseFactory< ALUConformGrid< 2, dimw > > ::
+           generateALU2dGrid( simplex, file, communicator, filename );
   }
 } // end namespace Dune
-/** \endcond */
+  /** \endcond */
