@@ -1,5 +1,5 @@
-// -*- tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
-// vi: set et ts=8 sw=2 sts=2:
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
 
 #if HAVE_CONFIG_H
 #include "config.h"
@@ -40,59 +40,59 @@ public:
   { }
 
   //! return number of components
-  virtual int ncomps () const  { return n; }
-  
+  virtual int ncomps () const { return n; }
+
   //! evaluate single component comp in the entity e at local coordinates xi
   /*! Evaluate the function in an entity at local coordinates.
-    @param[in]  comp   number of component to be evaluated
-    @param[in]  e      reference to grid entity of codimension 0
-    @param[in]  xi     point in local coordinates of the reference element of e
-    \return            value of the component
-  */
+     @param[in]  comp   number of component to be evaluated
+     @param[in]  e      reference to grid entity of codimension 0
+     @param[in]  xi     point in local coordinates of the reference element of e
+     \return            value of the component
+   */
   virtual double evaluate (int comp, const Entity& e, const Dune::FieldVector<DT,n>& xi) const
-    {
-      return comp*0.1;
-    }
+  {
+    return comp*0.1;
+  }
 
   // get name
   virtual std::string name () const
-    {
-      char _name[256];
-      snprintf(_name, 256, "%s-vector-%iD", type, ncomps());
-      return std::string(_name);
-    }
+  {
+    char _name[256];
+    snprintf(_name, 256, "%s-vector-%iD", type, ncomps());
+    return std::string(_name);
+  }
+
 
-  
 };
 
 template< class GridView >
 void doWrite( const GridView &gridView )
 {
-   enum { dim = GridView :: dimension };
+  enum { dim = GridView :: dimension };
 
-   const typename GridView :: IndexSet &is = gridView.indexSet();
-   std::vector<int> vertexdata(is.size(dim),dim);
-   std::vector<int> celldata(is.size(0),0);
+  const typename GridView :: IndexSet &is = gridView.indexSet();
+  std::vector<int> vertexdata(is.size(dim),dim);
+  std::vector<int> celldata(is.size(0),0);
 
-   Dune::VTK::ConformingVolumeWriter< GridView > vtk( gridView );
-   vtk.addVertexData(vertexdata,"vertexData");
-   vtk.addCellData(celldata,"cellData");
+  Dune::VTK::ConformingVolumeWriter< GridView > vtk( gridView );
+  vtk.addVertexData(vertexdata,"vertexData");
+  vtk.addCellData(celldata,"cellData");
 
-   vtk.addVertexData(new VTKVectorFunction<GridView>("vertex"));
-   vtk.addCellData(new VTKVectorFunction<GridView>("cell"));
+  vtk.addVertexData(new VTKVectorFunction<GridView>("vertex"));
+  vtk.addCellData(new VTKVectorFunction<GridView>("cell"));
 
-   char name[256];
-   snprintf(name,256,"conformvolumevtktest-%iD-ascii", dim);
-   vtk.write(name, Dune::VTK::ascii);
+  char name[256];
+  snprintf(name,256,"conformvolumevtktest-%iD-ascii", dim);
+  vtk.write(name, Dune::VTK::ascii);
 
-   snprintf(name,256,"conformvolumevtktest-%iD-base64", dim);
-   vtk.write(name, Dune::VTK::base64);
+  snprintf(name,256,"conformvolumevtktest-%iD-base64", dim);
+  vtk.write(name, Dune::VTK::base64);
 
-   snprintf(name,256,"conformvolumevtktest-%iD-appendedraw", dim);
-   vtk.write(name, Dune::VTK::appendedraw);
+  snprintf(name,256,"conformvolumevtktest-%iD-appendedraw", dim);
+  vtk.write(name, Dune::VTK::appendedraw);
 
-   snprintf(name,256,"conformvolumevtktest-%iD-appendedbase64", dim);
-   vtk.write(name, Dune::VTK::appendedbase64);
+  snprintf(name,256,"conformvolumevtktest-%iD-appendedbase64", dim);
+  vtk.write(name, Dune::VTK::appendedbase64);
 }
 
 template<int dim>
@@ -103,9 +103,9 @@ void vtkCheck(int* n, double* h)
   Dune::SGrid<dim,dim> g(n, h);
   g.globalRefine(1);
 
-  doWrite( g.template leafView< VTK_Partition >() );
-  doWrite( g.template levelView< VTK_Partition >( 0 ) );
-  doWrite( g.template levelView< VTK_Partition >( g.maxLevel() ) );
+  doWrite( g.template leafGridView< VTK_Partition >() );
+  doWrite( g.template levelGridView< VTK_Partition >( 0 ) );
+  doWrite( g.template levelGridView< VTK_Partition >( g.maxLevel() ) );
 }
 
 int main(int argc, char **argv)
@@ -122,6 +122,9 @@ int main(int argc, char **argv)
   } catch (Dune::Exception &e) {
     std::cerr << e << std::endl;
     return 1;
+  } catch (std::exception &e) {
+    std::cerr << e.what() << std::endl;
+    return 1;
   } catch (...) {
     std::cerr << "Generic exception!" << std::endl;
     return 2;
