@@ -1,4 +1,8 @@
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
 #include "config.h"
+#include <iostream>
+#include <ostream>
 #include <dune/grid/sgrid.hh>
 #include <dune/grid/io/file/gnuplot.hh>
 
@@ -18,11 +22,11 @@ void testIO(const GV & gridView, std::string fname)
 
   // create writer
   Dune::GnuplotWriter<GV> gnuplot(gridView);
-  
+
   // register data
   gnuplot.template addVertexData< std::vector<float> >(vertexdata, std::string("vertexdata"));
   gnuplot.template addCellData(celldata, std::string("celldata"));
-  
+
   // write data
   gnuplot.write(fname);
 }
@@ -33,11 +37,11 @@ int main()
     int n[] = { 10 };
     double h[] = { 1.0 };
     Dune::SGrid<1,1> sgrid(n,h);
-    
-    testIO(sgrid.leafView<Dune::InteriorBorder_Partition>(), "sgrid-leaf.data");
-    testIO(sgrid.levelView<Dune::InteriorBorder_Partition>(0), "sgrid-level.data");
-  } catch (Dune::Exception &e) {
-    std::cerr << e << std::endl;
+
+    testIO(sgrid.leafGridView<Dune::InteriorBorder_Partition>(), "sgrid-leaf.data");
+    testIO(sgrid.levelGridView<Dune::InteriorBorder_Partition>(0), "sgrid-level.data");
+  } catch (std::exception &e) {
+    std::cerr << e.what() << std::endl;
     return 1;
   } catch (...) {
     std::cerr << "Generic exception!" << std::endl;
