@@ -1,5 +1,5 @@
-// -*- tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
-// vi: set et ts=8 sw=2 sts=2:
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
 
 #if HAVE_CONFIG_H
 #include "config.h"
@@ -24,7 +24,7 @@ class ScalarFunction
 {
 public:
   typedef Dune::VTK::SkeletonFunctionTraits<GridView, typename GridView::ctype>
-     Traits;
+  Traits;
 
   //! return number of components
   unsigned dimRange() const { return 1; }
@@ -42,7 +42,7 @@ class VectorFunction
 {
 public:
   typedef Dune::VTK::SkeletonFunctionTraits<GridView, typename GridView::ctype>
-     Traits;
+  Traits;
 
   //! return number of components
   unsigned dimRange() const { return GridView::dimensionworld; }
@@ -52,7 +52,7 @@ public:
                 typename Traits::Range& result) const
   {
     const Dune::FieldVector<typename Traits::DomainField,
-      GridView::dimensionworld>& normal = c.unitOuterNormal(xl);
+        GridView::dimensionworld>& normal = c.unitOuterNormal(xl);
     result.assign(normal.begin(), normal.end());
   }
 
@@ -61,32 +61,32 @@ public:
 template< class GridView >
 void doWrite( const GridView &gridView )
 {
-   enum { dim = GridView :: dimension };
+  enum { dim = GridView :: dimension };
 
-   Dune::VTK::NonConformingBoundaryWriter< GridView > vtk( gridView );
+  Dune::VTK::NonConformingBoundaryWriter< GridView > vtk( gridView );
 
-   Dune::shared_ptr<ScalarFunction<GridView> > scalarFunc
-     (new ScalarFunction<GridView>);
-   vtk.addCellData(scalarFunc, "cellScalar");
-   vtk.addPointData(scalarFunc, "pointScalar");
+  Dune::shared_ptr<ScalarFunction<GridView> > scalarFunc
+    (new ScalarFunction<GridView>);
+  vtk.addCellData(scalarFunc, "cellScalar");
+  vtk.addPointData(scalarFunc, "pointScalar");
 
-   Dune::shared_ptr<VectorFunction<GridView> > vectorFunc
-     (new VectorFunction<GridView>);
-   vtk.addCellData(vectorFunc, "cellVector");
-   vtk.addPointData(vectorFunc, "pointVector");
+  Dune::shared_ptr<VectorFunction<GridView> > vectorFunc
+    (new VectorFunction<GridView>);
+  vtk.addCellData(vectorFunc, "cellVector");
+  vtk.addPointData(vectorFunc, "pointVector");
 
-   char name[256];
-   snprintf(name,256,"nonconformboundaryvtktest-%iD-ascii", dim);
-   vtk.write(name, Dune::VTK::ascii);
+  char name[256];
+  snprintf(name,256,"nonconformboundaryvtktest-%iD-ascii", dim);
+  vtk.write(name, Dune::VTK::ascii);
 
-   snprintf(name,256,"nonconformboundaryvtktest-%iD-base64", dim);
-   vtk.write(name, Dune::VTK::base64);
+  snprintf(name,256,"nonconformboundaryvtktest-%iD-base64", dim);
+  vtk.write(name, Dune::VTK::base64);
 
-   snprintf(name,256,"nonconformboundaryvtktest-%iD-appendedraw", dim);
-   vtk.write(name, Dune::VTK::appendedraw);
+  snprintf(name,256,"nonconformboundaryvtktest-%iD-appendedraw", dim);
+  vtk.write(name, Dune::VTK::appendedraw);
 
-   snprintf(name,256,"nonconformboundaryvtktest-%iD-appendedbase64", dim);
-   vtk.write(name, Dune::VTK::appendedbase64);
+  snprintf(name,256,"nonconformboundaryvtktest-%iD-appendedbase64", dim);
+  vtk.write(name, Dune::VTK::appendedbase64);
 }
 
 template<int dim>
@@ -97,9 +97,9 @@ void vtkCheck(int* n, double* h)
   Dune::SGrid<dim,dim> g(n, h);
   g.globalRefine(1);
 
-  doWrite( g.template leafView< VTK_Partition >() );
-  doWrite( g.template levelView< VTK_Partition >( 0 ) );
-  doWrite( g.template levelView< VTK_Partition >( g.maxLevel() ) );
+  doWrite( g.template leafGridView< VTK_Partition >() );
+  doWrite( g.template levelGridView< VTK_Partition >( 0 ) );
+  doWrite( g.template levelGridView< VTK_Partition >( g.maxLevel() ) );
 }
 
 int main(int argc, char **argv)
@@ -115,6 +115,9 @@ int main(int argc, char **argv)
   } catch (Dune::Exception &e) {
     std::cerr << e << std::endl;
     return 1;
+  } catch (std::exception &e) {
+    std::cerr << e.what() << std::endl;
+    return 1;
   } catch (...) {
     std::cerr << "Generic exception!" << std::endl;
     return 2;
