@@ -1,5 +1,5 @@
-// -*- tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
-// vi: set et ts=8 sw=2 sts=2:
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
 // $Id:$
 
 #if HAVE_CONFIG_H
@@ -14,7 +14,7 @@
 #include <unistd.h>
 
 #include <dune/common/fvector.hh>
-#include <dune/common/mpihelper.hh>
+#include <dune/common/parallel/mpihelper.hh>
 
 #include <dune/grid/io/file/vtk/vtkwriter.hh>
 #include <dune/grid/yaspgrid.hh>
@@ -23,9 +23,9 @@ const char* VTKDataMode(Dune::VTK::DataMode dm)
 {
   switch(dm)
   {
-  case Dune::VTK::conforming:
+  case Dune::VTK::conforming :
     return "conforming";
-  case Dune::VTK::nonconforming:
+  case Dune::VTK::nonconforming :
     return "nonconforming";
   }
   return "";
@@ -33,7 +33,7 @@ const char* VTKDataMode(Dune::VTK::DataMode dm)
 
 template< class GridView >
 class VTKVectorFunction
-: public Dune :: VTKWriter< GridView > :: VTKFunction
+  : public Dune :: VTKWriter< GridView > :: VTKFunction
 {
   // extract types
   enum { n = GridView :: dimension };
@@ -52,28 +52,28 @@ public:
   { }
 
   //! return number of components
-  virtual int ncomps () const  { return n; }
+  virtual int ncomps () const { return n; }
 
   //! evaluate single component comp in the entity e at local coordinates xi
   /*! Evaluate the function in an entity at local coordinates.
-    @param[in]  comp   number of component to be evaluated
-    @param[in]  e      reference to grid entity of codimension 0
-    @param[in]  xi     point in local coordinates of the reference element of e
-    \return            value of the component
-  */
+     @param[in]  comp   number of component to be evaluated
+     @param[in]  e      reference to grid entity of codimension 0
+     @param[in]  xi     point in local coordinates of the reference element of e
+     \return            value of the component
+   */
   virtual double evaluate (int comp, const Entity& e,
                            const Dune::FieldVector<DT,n>& xi) const
-    {
-      return comp*0.1;
-    }
+  {
+    return comp*0.1;
+  }
 
   // get name
   virtual std::string name () const
-    {
-      char _name[256];
-      snprintf(_name, 256, "%s-vector-%iD", type, ncomps());
-      return std::string(_name);
-    }
+  {
+    char _name[256];
+    snprintf(_name, 256, "%s-vector-%iD", type, ncomps());
+    return std::string(_name);
+  }
 
 };
 
@@ -118,23 +118,23 @@ void vtkCheck(const Dune::MPIHelper &mpiHelper, int* n, double* h)
   typedef Dune::YaspGrid<dim> Grid;
   Dune::FieldVector<typename Grid::ctype, dim> L(0);
   std::copy(h, h+dim, L.begin());
-  Dune::FieldVector<int, dim> s(0);
+  Dune::array<int, dim> s;
   std::copy(n, n+dim, s.begin());
 
   Dune::YaspGrid<dim> g(mpiHelper.getCommunicator(), L, s,
-                        Dune::FieldVector<bool, dim>(false), 0);
+                        std::bitset<dim>(), 0);
   g.globalRefine(1);
 
-  doWrite( g.template leafView< VTK_Partition >(), Dune::VTK::conforming );
-  doWrite( g.template leafView< VTK_Partition >(), Dune::VTK::nonconforming );
-  doWrite( g.template levelView< VTK_Partition >( 0 ),
-                                                   Dune::VTK::conforming );
-  doWrite( g.template levelView< VTK_Partition >( 0 ),
-                                                   Dune::VTK::nonconforming );
-  doWrite( g.template levelView< VTK_Partition >( g.maxLevel() ),
-                                                   Dune::VTK::conforming );
-  doWrite( g.template levelView< VTK_Partition >( g.maxLevel() ),
-                                                   Dune::VTK::nonconforming );
+  doWrite( g.template leafGridView< VTK_Partition >(), Dune::VTK::conforming );
+  doWrite( g.template leafGridView< VTK_Partition >(), Dune::VTK::nonconforming );
+  doWrite( g.template levelGridView< VTK_Partition >( 0 ),
+           Dune::VTK::conforming );
+  doWrite( g.template levelGridView< VTK_Partition >( 0 ),
+           Dune::VTK::nonconforming );
+  doWrite( g.template levelGridView< VTK_Partition >( g.maxLevel() ),
+           Dune::VTK::conforming );
+  doWrite( g.template levelGridView< VTK_Partition >( g.maxLevel() ),
+           Dune::VTK::nonconforming );
 }
 
 int main(int argc, char **argv)
@@ -156,6 +156,9 @@ int main(int argc, char **argv)
   } catch (Dune::Exception &e) {
     std::cerr << e << std::endl;
     return 1;
+  } catch (std::exception &e) {
+    std::cerr << e.what() << std::endl;
+    return 1;
   } catch (...) {
     std::cerr << "Generic exception!" << std::endl;
     return 2;
