@@ -1,3 +1,5 @@
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
 #include <iostream>
 #include <fstream>
 #include <vector>
@@ -12,12 +14,12 @@
  *
  *  For a fixed codimension c and a fixed upwind direction u, the test works
  *  as follows:
- *    1) In the center of all upwind codim c subentities of the interioir codim 0 
+ *    1) In the center of all upwind codim c subentities of the interioir codim 0
  *       leaf entites a function is stored. Also a flag is set to 1.
- *       The computation is also performed on the subentities of the physical 
+ *       The computation is also performed on the subentities of the physical
  *       boundary.
  *    -> For all leaf subentities of codim c the flag should be set to 1
- *       with the exception of the border subentities on the inflow 
+ *       with the exception of the border subentities on the inflow
  *       processor boundary and in the ghost elements - on these
  *       subentities the flag is zero.
  *    2) Exchange both the data and the flags.
@@ -31,12 +33,12 @@
 
 
 /*****
-  The exchange is done using the ExampleDataHandle given below.
-  Together with the function value and the flag the coordinates
-  of all corners of the subenties are transmitted, giving 
-  the possibility for additional testing in the scatter/set
-  methods.
-******/
+   The exchange is done using the ExampleDataHandle given below.
+   Together with the function value and the flag the coordinates
+   of all corners of the subenties are transmitted, giving
+   the possibility for additional testing in the scatter/set
+   methods.
+ ******/
 /*******************************************************************/
 namespace
 {
@@ -58,11 +60,11 @@ namespace
 }
 
 
-template <class IndexSetImp, 
-          class GlobalIdSetImp,
-          class DataVectorType >
-class ExampleDataHandle 
-: public Dune::CommDataHandleIF< ExampleDataHandle< IndexSetImp, GlobalIdSetImp, DataVectorType >, typename DataVectorType::value_type >
+template <class IndexSetImp,
+    class GlobalIdSetImp,
+    class DataVectorType >
+class ExampleDataHandle
+  : public Dune::CommDataHandleIF< ExampleDataHandle< IndexSetImp, GlobalIdSetImp, DataVectorType >, typename DataVectorType::value_type >
 {
   const IndexSetImp & iset_;
   const GlobalIdSetImp & ids_;
@@ -71,45 +73,46 @@ class ExampleDataHandle
   DataVectorType & data2_;
 public:
   typedef typename DataVectorType :: value_type DataType;
-  ExampleDataHandle(const IndexSetImp & iset, 
+  ExampleDataHandle(const IndexSetImp & iset,
                     const GlobalIdSetImp & ids,
-                    int cdim, 
+                    int cdim,
                     DataVectorType & d1, DataVectorType & d2) :
-    iset_(iset), ids_(ids) , cdim_(cdim), data1_(d1) , data2_(d2) 
+    iset_(iset), ids_(ids) , cdim_(cdim), data1_(d1) , data2_(d2)
   {}
 
   //! returns true if data for this codim should be communicated
   bool contains (int dim, int codim) const
   {
-     return (codim==cdim_);
+    return (codim==cdim_);
   }
-  
+
   //! returns true if size per entity of given dim and codim is a constant
   bool fixedsize (int dim, int codim) const
   {
-    // this problem is a fixed size problem, 
-    // but to simulate also non-fixed size problems 
-    // we set this to false, should work anyway 
+    // this problem is a fixed size problem,
+    // but to simulate also non-fixed size problems
+    // we set this to false, should work anyway
     return false;
   }
-  
+
   /*! how many objects of type DataType have to be sent for a given entity
-  Note: Only the sender side needs to know this size. 
-  */
+     Note: Only the sender side needs to know this size.
+   */
   template<class EntityType>
   size_t size (EntityType& e) const
   {
     // flag+data+coordinates
-    return 2+e.geometry().corners()*e.geometry().dimensionworld;
+    typedef typename EntityType::Geometry Geometry;
+    return 2+e.geometry().corners() * Geometry::dimensionworld;
   }
-  
+
   //! pack data from user to message buffer
   template<class MessageBuffer, class EntityType>
   void gather (MessageBuffer& buff, const EntityType& e) const
   {
-    int idx = iset_.index(e); 
-    
-    //typename GlobalIdSetImp :: IdType id = ids_.id( e ); 
+    int idx = iset_.index(e);
+
+    //typename GlobalIdSetImp :: IdType id = ids_.id( e );
     //buff.write( id );
     buff.write(data2_[idx]);   // flag
     buff.write(data1_[idx]);   // data
@@ -127,45 +130,45 @@ public:
   }
 
   /*! unpack data from message buffer to user
-  n is the number of objects sent by the sender
-  */
+     n is the number of objects sent by the sender
+   */
   template<class MessageBuffer, class EntityType>
   void scatter (MessageBuffer& buff, const EntityType& e, size_t n)
-  { 
-    // as this problem is a fixed size problem we can check the sizes 
+  {
+    // as this problem is a fixed size problem we can check the sizes
     assert( n == size(e) );
-    
-    // make sure that global id on all processors are the same 
-    // here compare the id of the entity that sended the data with my id 
-    //typename GlobalIdSetImp :: IdType id; 
-    //buff.read( id ); 
-    //typename GlobalIdSetImp :: IdType myId = ids_.id( e ); 
+
+    // make sure that global id on all processors are the same
+    // here compare the id of the entity that sended the data with my id
+    //typename GlobalIdSetImp :: IdType id;
+    //buff.read( id );
+    //typename GlobalIdSetImp :: IdType myId = ids_.id( e );
     //std::cout << id << " id | my Id = " << myId << "\n";
     //assert( id == myId );
-    
-    // else do normal scatter 
-    int idx = iset_.index(e); 
+
+    // else do normal scatter
+    int idx = iset_.index(e);
     DataType x=0.0;
     buff.read(x); // flag
 
-    // for ghost entities x > 0 must be true 
+    // for ghost entities x > 0 must be true
     assert( ( e.partitionType() == Dune::GhostEntity ) ? (x>=0.0) : 1);
-    
-    if (x>=0) 
-    { // only overwrite existing data if flag = 1, i.e., 
+
+    if (x>=0)
+    { // only overwrite existing data if flag = 1, i.e.,
       // the sending processor acctually computed the value
-      data2_[idx] = x; 
+      data2_[idx] = x;
       x=0.;
       buff.read(x);  // correct function value
-      data1_[idx] = x; 
-    } 
-    else 
+      data1_[idx] = x;
+    }
+    else
     {
       x=0.;
       buff.read(x);
     }
 
-    // test if the sending/receving entities are geometrically the same 
+    // test if the sending/receving entities are geometrically the same
     typedef typename EntityType::Geometry Geometry;
     const Geometry &geometry = e.geometry();
     for( int i = 0; i < geometry.corners(); ++i )
@@ -178,12 +181,12 @@ public:
         if( std::abs( corner[ j ] - x ) > 1e-8 )
         {
           std::cerr << "ERROR in scatter: Vertex <" << i << "," << j << ">: "
-                    << " this : (" << corner[ j ] << ")" 
+                    << " this : (" << corner[ j ] << ")"
                     << " other : (" << x << ")"
                     << std::endl;
         }
       }
-    }  
+    }
   }
 
 };
@@ -199,13 +202,13 @@ class CheckCommunication
   typedef typename GridView :: IntersectionIterator IntersectionIterator;
 
   typedef typename IntersectionIterator :: Intersection Intersection;
-  
-  typedef typename GridView :: template Codim< 0 > :: EntityPointer EntityPointer; 
+
+  typedef typename GridView :: template Codim< 0 > :: EntityPointer EntityPointer;
   typedef typename GridView :: template Codim< 0 > :: Entity Entity;
   typedef typename GridView :: template Codim< 0 > :: Iterator Iterator;
 
-  typedef typename GridView :: template Codim< cdim > :: EntityPointer SubEntityPointer; 
-  
+  typedef typename GridView :: template Codim< cdim > :: EntityPointer SubEntityPointer;
+
   enum { dimworld = Grid :: dimensionworld };
   enum { dim = Grid :: dimension };
 
@@ -213,13 +216,13 @@ class CheckCommunication
 
   typedef Dune::FieldVector< ctype, dimworld > CoordinateVector;
   typedef std :: vector< double > ArrayType;
-  
+
   CoordinateVector upwind_;
   OutputStream &sout_;
 
   const GridView &gridView_;
   const IndexSet &indexSet_;
-  const int level_; 
+  const int level_;
 
   // the function
   double f ( const CoordinateVector &x )
@@ -228,22 +231,22 @@ class CheckCommunication
     a[0] = -0.5;
     return a*x+1.5; //+cos(x*x);
   }
-  
+
   // compute the data on the upwind entities
   void project ( int dataSize, ArrayType &data, ArrayType &weight, int rank )
   {
-    // set initial data 
-    for(int i=0 ;i<dataSize; ++i) 
+    // set initial data
+    for(int i=0 ; i<dataSize; ++i)
     {
       data[i] = 0.0;
       weight[i] = -1.0;
     }
-    
+
     Iterator end = gridView_.template end< 0 >();
     for( Iterator it = gridView_.template begin< 0 >(); it != end ; ++it )
     {
       const Entity &entity = *it;
-      
+
       if( cdim == 0 )
       {
         CoordinateVector mid( 0.0 );
@@ -252,11 +255,11 @@ class CheckCommunication
           mid += entity.geometry().corner( i );
         mid /= double( numVertices );
 
-        int index = indexSet_.index( entity ); 
+        int index = indexSet_.index( entity );
         data[ index ]  = f( mid );
         weight[ index ] = 1.0;
-      } 
-      else 
+      }
+      else
       {
         const IntersectionIterator nend = gridView_.iend( entity );
         for( IntersectionIterator nit = gridView_.ibegin( entity ); nit != nend; ++nit )
@@ -265,21 +268,21 @@ class CheckCommunication
 
           const typename Intersection::LocalGeometry &geoInSelf = intersection.geometryInInside();
 
-          const Dune::GenericReferenceElement< ctype, dim-1 > &faceRefElement
-            = Dune::GenericReferenceElements< ctype, dim-1 > :: general( geoInSelf.type() );
+          const Dune::ReferenceElement< ctype, dim-1 > &faceRefElement
+            = Dune::ReferenceElements< ctype, dim-1 > :: general( geoInSelf.type() );
           const Dune::FieldVector< ctype, dim-1 > &bary = faceRefElement.position( 0, 0 );
-          
+
           const CoordinateVector normal = intersection.integrationOuterNormal( bary );
           double calc = normal * upwind_;
-        
+
           // if testing level, then on non-conform grid also set values on
           // intersections that are not boundary, but has no level
-          // neighbor 
+          // neighbor
           const bool proceedAnyway = (level_ < 0 ? false : !intersection.neighbor());
           if( (calc > -1e-8) || intersection.boundary() || proceedAnyway )
           {
-            const Dune::GenericReferenceElement< ctype, dim > &insideRefElem
-              = Dune::GenericReferenceElements< ctype, dim >::general( entity.type() );
+            const Dune::ReferenceElement< ctype, dim > &insideRefElem
+              = Dune::ReferenceElements< ctype, dim >::general( entity.type() );
 
             const int indexInInside = intersection.indexInInside();
             for( int i = 0; i < insideRefElem.size( indexInInside, 1, cdim ); ++i )
@@ -295,21 +298,21 @@ class CheckCommunication
 
               data[ idx ] = f( cmid );
               weight[ idx ] = 1.0;
-            } 
-            
+            }
+
             // on non-conforming grids the neighbor entities might not
             // be the same as those on *it, therefore set data on neighbor
             // as well
-            if( intersection.neighbor() ) 
+            if( intersection.neighbor() )
             {
               EntityPointer ep = intersection.outside();
               const Entity &neigh = *ep;
-             
+
               assert( (level_ < 0) ? (neigh.isLeaf()) : 1);
               assert( (level_ < 0) ? 1 : (neigh.level() == level_) );
-            
-              const Dune::GenericReferenceElement< ctype, dim > &outsideRefElem
-                = Dune::GenericReferenceElements< ctype, dim >::general( neigh.type() );
+
+              const Dune::ReferenceElement< ctype, dim > &outsideRefElem
+                = Dune::ReferenceElements< ctype, dim >::general( neigh.type() );
 
               const int indexInOutside = intersection.indexInOutside();
               for( int i = 0; i < outsideRefElem.size( indexInOutside, 1, cdim ); ++i )
@@ -329,7 +332,7 @@ class CheckCommunication
             }
           }
         }
-      } 
+      }
     }
   }
 
@@ -341,19 +344,19 @@ class CheckCommunication
   {
     const int rank = gridView_.comm().rank();
     const int size = gridView_.comm().size();
-    
+
     // check whether there is an overlap or ghost region of
     // cells for the current grid view.
-    if (size > 1 && 
-        gridView_.overlapSize(0) == 0 && 
+    if (size > 1 && cdim == 0 &&
+        gridView_.overlapSize(0) == 0 &&
         gridView_.ghostSize(0) == 0)
     {
-        sout_ << "<" << rank << "/test> Error in communication test.";
-        sout_ << " overlapSize+ghostSize:" << gridView_.overlapSize(0) + gridView_.ghostSize(0) << " (should not be 0)";
-        sout_ << " communicator size is:" << size;
-        sout_ << std :: endl;
+      sout_ << "<" << rank << "/test> Error in communication test.";
+      sout_ << " overlapSize+ghostSize:" << gridView_.overlapSize(0) + gridView_.ghostSize(0) << " (should not be 0)";
+      sout_ << " communicator size is:" << size;
+      sout_ << std :: endl;
     }
-    
+
     //Variante MIT Geisterzellen
     //typedef typename IndexSet :: template Codim<0> :: template Partition<All_Partition> :: Iterator IteratorType;
 
@@ -362,20 +365,20 @@ class CheckCommunication
     for( Iterator it = gridView_.template begin< 0 >(); it != end ; ++it )
     {
       const Entity &entity = *it;
-      
+
       CoordinateVector mid( 0.0 );
       const int numVertices = entity.template count< dim >();
       for( int i = 0; i < numVertices; ++i )
         mid += entity.geometry().corner( i );
-      mid /= double(numVertices); 
-      
+      mid /= double(numVertices);
+
       if( cdim == 0 )
       {
         int index = indexSet_.index( entity );
         double lerr = std::abs( f( mid ) - data[ index ] );
         maxerr = std :: max( maxerr, lerr );
         if( testweight && (weight[ index ] < 0) )
-        { 
+        {
           sout_ << "<" << rank << "/test> Error in communication test.";
           sout_ << " weight:" << weight[ index ] << " (should be 0)";
           sout_ << " value is : " << data[ index ];
@@ -383,15 +386,15 @@ class CheckCommunication
           sout_ << " level:" << entity.level();
           sout_ << std :: endl;
         }
-      } 
-      else 
+      }
+      else
       {
         const int numSubEntities = entity.template count< cdim >();
         for( int i=0; i < numSubEntities; ++i )
         {
           SubEntityPointer subEp = entity.template subEntity< cdim >( i );
-          
-          const int index = indexSet_.index( *subEp ); 
+
+          const int index = indexSet_.index( *subEp );
           CoordinateVector cmid( 0.0 );
 
           const int numVertices = subEp->geometry().corners();
@@ -409,24 +412,24 @@ class CheckCommunication
             sout_ << " index is:" << index;
             sout_ << " level: " << entity.level() ;
             sout_ << std :: endl;
-            
+
             for( int j = 0; j < numVertices; )
             {
-              const Dune::GenericReferenceElement< double, dim > &refElem
-                = Dune::GenericReferenceElements< double, dim >::general( entity.type() );
+              const Dune::ReferenceElement< double, dim > &refElem
+                = Dune::ReferenceElements< double, dim >::general( entity.type() );
               const int vx = refElem.subEntity( i, cdim, j, dim );
 
               sout_ << "index: " << indexSet_.subIndex( entity, vx, dim )
                     << " " << subEp->geometry().corner( j );
               (++j < numVertices ? sout_ <<  "/" : sout_ << std :: endl);
             }
-          }  
-        } 
+          }
+        }
       }
     }
     return maxerr;
   }
-  
+
   // The main ''algorithm''
   bool checkCommunication ()
   {
@@ -444,15 +447,15 @@ class CheckCommunication
     ArrayType data(dataSize, 0.0);
     ArrayType weight(dataSize, 0.0);
     project( dataSize, data, weight, myrank );
-    
+
     double preresult = test( dataSize, data, weight, false );
     sout_ << "Test before Communication on <" << myrank << "> " << preresult << std :: endl;
     // Communicate
     typedef typename Grid :: Traits :: GlobalIdSet GlobalIdSet;
     ExampleDataHandle< IndexSet, GlobalIdSet, ArrayType >
-      dh( indexSet_, gridView_.grid().globalIdSet(), cdim, data, weight );
+    dh( indexSet_, gridView_.grid().globalIdSet(), cdim, data, weight );
 
-    // call communication of grid 
+    // call communication of grid
     try
     {
       gridView_.communicate(dh,Dune::InteriorBorder_All_Interface,Dune::ForwardCommunication);
@@ -475,24 +478,31 @@ class CheckCommunication
     sout_ << "Test after Communication on <" << myrank << "> " << result << std :: endl;
     return (std::abs(result) < 1e-8);
   }
-  
+
 public:
   CheckCommunication ( const GridView &gridView, OutputStream &sout, int level )
-  : upwind_( -1.0 ),
-    sout_( sout ),
-    gridView_( gridView ),
-    indexSet_( gridView_.indexSet() ),
-    level_( level )
+    : upwind_( -1.0 ),
+      sout_( sout ),
+      gridView_( gridView ),
+      indexSet_( gridView_.indexSet() ),
+      level_( level )
   {
-    if( !checkCommunication() )
+    // if no overlap and ghost is available we skip the check
+    const bool skipCheck = ( cdim == 0 ) ? (gridView_.overlapSize(0) == 0 && gridView_.ghostSize(0) == 0) : false ;
+
+    if( skipCheck )
+    {
+      std :: cerr << "Codim " << cdim << ": Test skiped because of empty set of overlap and ghosts !" << std :: endl;
+    }
+    else if ( ! checkCommunication() )
     {
       std :: cerr << "Error in communication test for codim "
                   << cdim << "!" << std :: endl;
     }
-    
+
     // for automatic testing of all codims
     CheckCommunication< GridView, NextCodim< Grid, cdim >::v, OutputStream >
-      test( gridView_, sout_, level_ );
+    test( gridView_, sout_, level_ );
   }
 };
 
@@ -514,13 +524,13 @@ void checkCommunication( const Grid &grid, int level, OutputStream &sout )
     typedef typename Grid::template Partition< Dune::All_Partition >::LeafGridView GridView;
     GridView gridView = grid.leafView();
     CheckCommunication< GridView, NextCodim< Grid >::v, OutputStream >
-      test( gridView, sout, level );
+    test( gridView, sout, level );
   }
-  else 
+  else
   {
     typedef typename Grid::template Partition< Dune::All_Partition >::LevelGridView GridView;
-    GridView gridView = grid.levelView( level );
+    GridView gridView = grid.levelGridView( level );
     CheckCommunication< GridView, NextCodim< Grid >::v, OutputStream >
-      test( gridView, sout, level );
+    test( gridView, sout, level );
   }
 }
