@@ -1,29 +1,32 @@
-// $Id: gridcheck.cc 8761 2013-01-19 15:09:31Z mblatt $
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
+// $Id$
 #ifndef DUNE_GRIDCHECK_CC
 #define DUNE_GRIDCHECK_CC
 
 /** \file
     \brief Implements a generic grid check
 
-  
-  \todo check return types
-  
-*/
+
+   \todo check return types
+
+ */
 
 #include <dune/common/exceptions.hh>
 #include <dune/common/stdstreams.hh>
 #include <dune/geometry/referenceelements.hh>
-#include <dune/geometry/genericgeometry/conversion.hh>
 #include <dune/grid/common/gridinfo.hh>
 #include <dune/grid/common/capabilities.hh>
 
 #include "staticcheck.hh"
 #include "checkindexset.cc"
 #include "checkgeometry.cc"
+#include "checkentityseed.cc"
+
 
 #include <limits>
 
-// machine epsilon is multiplied by this factor 
+// machine epsilon is multiplied by this factor
 static double factorEpsilon = 1.e8;
 
 class CheckError : public Dune::Exception {};
@@ -54,7 +57,7 @@ struct subIndexCheck
     const int imax = e.template count<cd>();
     for( int i = 0; i < imax; ++i )
     {
-      // check construction of entity pointers   
+      // check construction of entity pointers
       EntityPointer ep( *(e.template subEntity< cd >( i ) ) );
       assert( ep == e.template subEntity< cd >( i ) );
 
@@ -67,8 +70,8 @@ struct subIndexCheck
       #endif
       assert( ep == ep2 );
 
-      const typename Grid::LevelGridView &levelGridView = g.levelView(e.level());
-      
+      const typename Grid::LevelGridView &levelGridView = g.levelGridView(e.level());
+
       if( !levelGridView.contains( *ep ) )
       {
         std::cerr << "Error: Level grid view does not contain all subentities." << std::endl;
@@ -96,26 +99,6 @@ struct subIndexCheck
         assert( false );
       }
 
-#if 0  // should this be removed?
-      typedef Dune::GenericGeometry::MapNumberingProvider< Entity::dimension > Numbering;
-      const unsigned int tid = Dune::GenericGeometry::topologyId( e.type() );
-      const int oldi = Numbering::template generic2dune< cd >( tid, i );
-
-      if( levelIndexSet.subIndex( e, i, cd ) != levelIndexSet.template subIndex< cd >( e, oldi ) )
-      {
-        int id_e = levelIndexSet.index( e );
-        int subid_e_i = levelIndexSet.template subIndex< cd >( e, oldi );
-        int subid_e_i_cd = levelIndexSet.subIndex( e, i, cd );
-
-        std::cerr << "Error: levelIndexSet.subIndex( e, i, cd ) "
-                  << "!= levelIndexSet.subIndex< cd >( e, generic2dune( i ) )  "
-                  << "[with cd=" << cd << ", i=" << i << "]" << std::endl;
-        std::cerr << "       ... index(e)=" << id_e << std::endl;
-        std::cerr << "       ... subIndex<cd>(e,i)=" << subid_e_i << std::endl;
-        std::cerr << "       ... subIndex(e,dune2generic(i),cd)=" << subid_e_i_cd << std::endl;
-        assert( false );
-      }
-#endif
     }
 
     subIndexCheck< cd-1, Grid, Entity, Dune::Capabilities::hasEntity< Grid, cd-1 >::v > sick( g, e );
@@ -128,27 +111,27 @@ template <class Grid, class Entity, bool doCheck>
 struct subIndexCheck<-1, Grid, Entity, doCheck>
 {
   subIndexCheck (const Grid & g, const Entity & e)
-    {
-      return;
-    }
+  {
+    return;
+  }
 };
 // do nothing if doCheck==false
 template <int cd, class Grid, class Entity>
 struct subIndexCheck<cd, Grid, Entity, false>
 {
   subIndexCheck (const Grid & g, const Entity & e)
-    {
-      subIndexCheck<cd-1,Grid,Entity,
+  {
+    subIndexCheck<cd-1,Grid,Entity,
         Dune::Capabilities::hasEntity<Grid,cd-1>::v> sick(g,e);
-    }
+  }
 };
 template <class Grid, class Entity>
 struct subIndexCheck<-1, Grid, Entity, false>
 {
   subIndexCheck (const Grid & g, const Entity & e)
-    {
-      return;
-    }
+  {
+    return;
+  }
 };
 
 // name says all
@@ -171,7 +154,7 @@ void zeroEntityConsistency (Grid &g)
 
   LevelIterator it = g.template lbegin<0>(g.maxLevel());
   const LevelIterator endit = g.template lend<0>(g.maxLevel());
-  
+
   for (; it!=endit; ++it)
   {
     // check construction from entity
@@ -229,11 +212,11 @@ void zeroEntityConsistency (Grid &g)
       }
     }
 
-    // check that corner coordinates are distinct 
+    // check that corner coordinates are distinct
     const int corners= it->geometry().corners();
     for (int c=0; c<corners; ++c)
     {
-      for(int d=c+1; d<corners; ++d) 
+      for(int d=c+1; d<corners; ++d)
       {
         const typename Geometry::GlobalCoordinate c1 = it->geometry().corner( c );
         const typename Geometry::GlobalCoordinate c2 = it->geometry().corner( d );
@@ -242,11 +225,11 @@ void zeroEntityConsistency (Grid &g)
       }
     }
 
-    // check hasSingleGeometryType capability 
+    // check hasSingleGeometryType capability
     const bool hasSingleGeomType = Dune :: Capabilities :: hasSingleGeometryType< Grid > :: v ;
-    if( hasSingleGeomType ) 
+    if( hasSingleGeomType )
     {
-      // check that geometry type generated from constant topo Id is the same as of the codim 0 entity  
+      // check that geometry type generated from constant topo Id is the same as of the codim 0 entity
       const Dune::GeometryType constantType( Dune :: Capabilities :: hasSingleGeometryType< Grid > :: topologyId , dimGrid );
       if( it->type() != constantType )
       {
@@ -255,10 +238,10 @@ void zeroEntityConsistency (Grid &g)
     }
 
     const bool isCartesian = Dune :: Capabilities :: isCartesian< Grid > :: v ;
-    // check geometry for Cartesian grids 
+    // check geometry for Cartesian grids
     if( isCartesian )
     {
-      if( ! it->geometry().affine() ) 
+      if( ! it->geometry().affine() )
         DUNE_THROW( Dune::GridError, "Geometry is not affine, although isCartesian is true");
     }
 
@@ -277,20 +260,20 @@ void assertNeighbor (Grid &g)
 
   typedef typename Grid::template Codim<0>::LevelIterator LevelIterator;
   enum { dim = Grid::dimension };
-  typedef typename Grid::ctype ct;
+  //typedef typename Grid::ctype ct DUNE_UNUSED;
 
   typedef typename Grid::GlobalIdSet GlobalIdSet;
   const GlobalIdSet & globalid = g.globalIdSet();
 
   typedef typename Grid::template Codim< 0 >::Entity Entity;
   typedef typename Grid::template Codim< 0 >::EntityPointer EntityPointer;
- 
-  GridView gridView = g.levelView( 0 );
+
+  GridView gridView = g.levelGridView( 0 );
 
   LevelIterator e = g.template lbegin<0>(0);
   const LevelIterator eend = g.template lend<0>(0);
 
-  LevelIterator next = e; 
+  LevelIterator next = e;
   if (next != eend)
   {
     ++next;
@@ -318,13 +301,13 @@ void assertNeighbor (Grid &g)
     }
 
     // iterate over level
-    for (;e != eend; ++e)
+    for (; e != eend; ++e)
     {
       const Entity &entity = *e;
 
       const bool isGhost = (entity.partitionType() == Dune::GhostEntity);
 
-      // call global id 
+      // call global id
       globalid.id( entity );
 
       if( !entity.isLeaf() )
@@ -379,9 +362,6 @@ void assertNeighbor (Grid &g)
         if( it->boundary() )
         {
           it->boundarySegmentIndex();
-#if !DISABLE_DEPRECATED_METHOD_CHECK
-          it->boundaryId();
-#endif // #if !DISABLE_DEPRECATED_METHOD_CHECK
         }
 
         // check id
@@ -391,7 +371,7 @@ void assertNeighbor (Grid &g)
         // geometry
         it->geometryInInside();
         it->geometry();
-        
+
         // normal vectors
         typename IntersectionIterator::Intersection::LocalCoordinate v(0);
 
@@ -404,7 +384,7 @@ void assertNeighbor (Grid &g)
           std::cerr << "Error: On ghosts, all intersections must have neighbors." << std::endl;
           assert( false );
         }
-       
+
         if( it->neighbor() )
         {
           const EntityPointer outsidePtr = it->outside();
@@ -415,7 +395,7 @@ void assertNeighbor (Grid &g)
             std::cerr << "Error: Intersections between ghosts shall not be returned." << std::endl;
             assert( false );
           }
-          
+
           // geometry
           it->geometryInOutside();
 
@@ -437,14 +417,14 @@ void assertNeighbor (Grid &g)
             const Dune::PartitionIteratorType pitype = Dune::InteriorBorder_Partition;
 
             typedef typename Grid::template Codim< 0 >
-              ::template Partition< pitype >::LevelIterator
-              LevelIterator;
+            ::template Partition< pitype >::LevelIterator
+            LevelIterator;
 
             const int level = entity.level();
             bool foundNeighbor = false;
             LevelIterator nit = g.template lbegin< 0, pitype >( level );
             const LevelIterator nend = g.template lend< 0, pitype > ( level );
-            for( ; nit != nend; ++nit ) 
+            for( ; nit != nend; ++nit )
             {
               if( nit->partitionType() != Dune::InteriorEntity )
               {
@@ -480,48 +460,47 @@ void assertNeighbor (Grid &g)
   }
 }
 
-template <class GridType, bool c> 
+template <class GridType, bool c>
 struct CheckMark
 {
   template <class IteratorType>
   static void check(GridType & grid, IteratorType & it)
   {
-      // last marker is 0, so the grid is not changed after this check
-      const int refCount[4] = {1,0,-1,0};
-      for(int k=0; k<4; ++k) 
+    // last marker is 0, so the grid is not changed after this check
+    const int refCount[4] = {1,0,-1,0};
+    for(int k=0; k<4; ++k)
+    {
+      // mark entity
+      bool marked = grid.mark( refCount[k] , *it );
+      // if element was marked, check that the marker was set correctly
+      if(marked)
       {
-        // mark entity 
-        bool marked = grid.mark( refCount[k] , *it );
-        // if element was marked, check that the marker was set correctly 
-        if(marked)
-        {
-          // now getMark should return the mark we just set, otherwise error
-          if( grid.getMark( *it ) < refCount[k] )
-            DUNE_THROW(CheckError,"mark/getMark method not working correctly!");
-        }
+        // now getMark should return the mark we just set, otherwise error
+        if( grid.getMark( *it ) < refCount[k] )
+          DUNE_THROW(CheckError,"mark/getMark method not working correctly!");
       }
+    }
   }
 };
 
-template <class GridType> 
+template <class GridType>
 struct CheckMark<GridType,false>
 {
   template <class IteratorType>
   static void check(const GridType & grid, IteratorType & it )
-  {
-  }
+  {}
 };
 
 /*
  * Iterate over the grid und do some runtime checks
  */
 
-template <bool checkMark , class Grid> 
+template <bool checkMark , class Grid>
 void iterate(Grid &g)
 {
   typedef typename Grid::template Codim<0>::LevelIterator LevelIterator;
-  typedef typename Grid::template Codim<0>::EntityPointer EntityPointer;
-  typedef typename Grid::HierarchicIterator HierarchicIterator;
+  //typedef typename Grid::template Codim<0>::EntityPointer EntityPointer DUNE_UNUSED;
+  //typedef typename Grid::HierarchicIterator HierarchicIterator DUNE_UNUSED;
   typedef typename Grid::template Codim<0>::Geometry Geometry;
   int l = g.maxLevel();
   LevelIterator it = g.template lbegin<0>(l);
@@ -530,7 +509,7 @@ void iterate(Grid &g)
   typename Geometry::LocalCoordinate origin(1);
   typename Geometry::LocalCoordinate result;
 
-  for (;it != endit; ++it)
+  for (; it != endit; ++it)
   {
     LevelIterator l1 = it;
     LevelIterator l2 = l1; ++l1;
@@ -540,14 +519,14 @@ void iterate(Grid &g)
     assert( (l1 == l2) && (l2 == l1) );
 
     const Geometry &geo = it->geometry();
-    
+
     result = geo.local( geo.global( origin ) );
     typename Grid::ctype error = (result-origin).two_norm();
     if(error >= factorEpsilon * std::numeric_limits<typename Grid::ctype>::epsilon())
-      {
-        DUNE_THROW(CheckError, "|| geom.local(geom.global(" << origin
-                   << ")) - origin || != 0 ( || " << result << " - origin || ) = " << error);
-      };
+    {
+      DUNE_THROW(CheckError, "|| geom.local(geom.global(" << origin
+                                                          << ")) - origin || != 0 ( || " << result << " - origin || ) = " << error);
+    };
     geo.integrationElement( origin );
 
     if((int)Geometry::coorddimension == (int)Geometry::mydimension)
@@ -562,15 +541,15 @@ void iterate(Grid &g)
     geo.corners();
     geo.corner( 0 );
   }
-  
+
   typedef typename Grid::template Codim<0>::LeafIterator LeafIterator;
   LeafIterator lit = g.template leafbegin<0>();
   const LeafIterator lend = g.template leafend<0>();
 
-  // if empty grid, do nothing 
+  // if empty grid, do nothing
   if(lit == lend) return;
 
-  for (;lit != lend; ++lit)
+  for (; lit != lend; ++lit)
   {
     LeafIterator l1 = lit;
     LeafIterator l2 = l1; ++l1;
@@ -579,20 +558,20 @@ void iterate(Grid &g)
     ++l2;
     assert(l1 == l2);
 
-    // leaf check 
-    if( !lit->isLeaf() ) 
+    // leaf check
+    if( !lit->isLeaf() )
       DUNE_THROW(CheckError,"LeafIterator gives non-leaf entity!");
 
-    // check adaptation mark for leaf entity mark 
+    // check adaptation mark for leaf entity mark
     CheckMark<Grid,checkMark>::check(g,lit);
 
     result = lit->geometry().local(lit->geometry().global(origin));
     typename Grid::ctype error = (result-origin).two_norm();
     if(error >= factorEpsilon * std::numeric_limits<typename Grid::ctype>::epsilon())
-      {
-        DUNE_THROW(CheckError, "|| geom.local(geom.global(" << origin
-                   << ")) - origin || != 0 ( || " << result << " - origin || ) = " << error);
-      };
+    {
+      DUNE_THROW(CheckError, "|| geom.local(geom.global(" << origin
+                                                          << ")) - origin || != 0 ( || " << result << " - origin || ) = " << error);
+    };
     lit->geometry().integrationElement(origin);
     if((int)Geometry::coorddimension == (int)Geometry::mydimension)
       lit->geometry().jacobianInverseTransposed(origin);
@@ -616,7 +595,7 @@ void iteratorEquals (Grid &g)
   typedef typename Grid::LeafGridView LeafGridView;
   typedef typename LeafGridView::IntersectionIterator LeafIntersectionIterator;
 
-  LeafGridView leafView = g.leafView();
+  LeafGridView leafView = g.leafGridView();
 
   // assignment tests
   LevelIterator l1 = g.template lbegin<0>(0);
@@ -624,14 +603,14 @@ void iteratorEquals (Grid &g)
   LeafIterator L1 = g.template leafbegin<0>();
   LeafIterator L2 = g.template leafbegin<0>();
 
-  // if grid empty, leave 
-  if (l1 == g.template lend<0>(0)) 
+  // if grid empty, leave
+  if (l1 == g.template lend<0>(0))
     return;
 
   // check '==' consistency
   EntityPointer a( g.template levelView<Dune::All_Partition>(0).template begin<0>() );
   EntityPointer i( g.template levelView<Dune::Interior_Partition>(0).template begin<0>() );
-  
+
   assert(
     (g.levelIndexSet(0).index(*a) != g.levelIndexSet(0).index(*i)) // index equal
     == (a != i) // entitypointer equal
@@ -657,13 +636,13 @@ void iteratorEquals (Grid &g)
 
   // equals
   #define TestEquals(i) { \
-          { bool DUNE_UNUSED tmp = (i == e2); }                         \
-          { bool DUNE_UNUSED tmp = (i == l2); }                         \
-          { bool DUNE_UNUSED tmp = (i == h2); }                         \
-          { bool DUNE_UNUSED tmp = (i == L2); }                         \
-          if (i2 != leafView.iend( *l2 )) bool DUNE_UNUSED tmp = (i == i2->inside()); \
-          if (i2 != leafView.iend( *l2 ) && i2->neighbor()) bool DUNE_UNUSED tmp = (i == i2->outside()); \
-  }
+    { bool DUNE_UNUSED tmp = (i == e2); }                         \
+    { bool DUNE_UNUSED tmp = (i == l2); }                         \
+    { bool DUNE_UNUSED tmp = (i == h2); }                         \
+    { bool DUNE_UNUSED tmp = (i == L2); }                         \
+    if (i2 != leafView.iend( *l2 )) bool DUNE_UNUSED tmp = (i == i2->inside()); \
+    if (i2 != leafView.iend( *l2 ) && i2->neighbor()) bool DUNE_UNUSED tmp = (i == i2->outside()); \
+}
   TestEquals(e1);
   TestEquals(l1);
   TestEquals(h1);
@@ -683,13 +662,13 @@ void checkBoundarySegmentIndexProlongation ( const Grid &grid, const Entity &ent
 
   const typename Intersection::LocalGeometry &geoInInside = intersection.geometryInInside();
 
-  const GridView gridView = grid.levelView( entity.level()+1 );
+  const GridView gridView = grid.levelGridView( entity.level()+1 );
   const HierarchicIterator hend = entity.hend( entity.level()+1 );
   for( HierarchicIterator hit = entity.hbegin( entity.level()+1 ); hit != hend; ++hit )
   {
     GeometryInFather geoInFather = hit->geometryInFather();
-    const Dune::GenericReferenceElement< typename Grid::ctype, Grid::dimension > &refElement
-      = Dune::GenericReferenceElements< typename Grid::ctype, Grid::dimension >::general( geoInFather.type() );
+    const Dune::ReferenceElement< typename Grid::ctype, Grid::dimension > &refElement
+      = Dune::ReferenceElements< typename Grid::ctype, Grid::dimension >::general( geoInFather.type() );
 
     const IntersectionIterator iend = gridView.iend( *hit );
     for( IntersectionIterator iit = gridView.ibegin( *hit ); iit != iend; ++iit )
@@ -714,6 +693,75 @@ void checkBoundarySegmentIndexProlongation ( const Grid &grid, const Entity &ent
 }
 
 
+// check for consistency of hasFather(), father(), and father().level()
+template< class Grid >
+void checkFatherLevel ( Grid &grid )
+{
+  for(int level=0; level<=grid.maxLevel(); ++level)
+  {
+    typedef typename Grid::LevelGridView GridView;
+    typedef typename GridView::template Codim<0>::Iterator Iterator;
+    typedef typename GridView::template Codim<0>::EntityPointer EntityPointer;
+
+    GridView gv = grid.levelGridView(level);
+    Iterator it = gv.template begin<0>();
+    Iterator end = gv.template end<0>();
+    for(; it!=end; ++it)
+    {
+      if (level==0)
+      {
+        if (it->hasFather())
+        {
+          std::cerr << "Error: hasFather() is true for element " << gv.indexSet().index(*it)
+                    << " on level 0"
+                    << std::endl;
+          assert(false);
+        }
+      }
+      else
+      {
+        if (it->hasFather())
+        {
+          // check level of father for newly created entitypointer
+          {
+            EntityPointer f = it->father();
+            if (f.level() != level-1)
+            {
+              std::cerr << "Error: father().level()=" << f.level()
+                        << " for element on level " << level << std::endl;
+              assert(false);
+            }
+            if (f->level() != level-1)
+            {
+              std::cerr << "Error: father()->level()=" << f.level()
+                        << " for element on level " << level << std::endl;
+              assert(false);
+            }
+          }
+          // check level of father after reassignment
+          {
+            EntityPointer f(*it);
+            f = it->father();
+            if (f.level() != level-1)
+            {
+              std::cerr << "Error: father().level()=" << f.level()
+                        << " for element on level " << level << " with reassigned father pointer" << std::endl;
+              assert(false);
+            }
+            if (f->level() != level-1)
+            {
+              std::cerr << "Error: father()->level()=" << f.level()
+                        << " for element on level " << level << " with reassigned father pointer" << std::endl;
+              assert(false);
+            }
+          }
+        }
+      }
+    }
+  }
+}
+
+
 template< class GridView >
 void checkBoundarySegmentIndex ( const GridView &gridView )
 {
@@ -778,7 +826,11 @@ void checkBoundarySegmentIndex ( const GridView &gridView )
 
   if (error)
   {
-    DUNE_THROW(Dune::Exception, "Error encountered during checkBoundarySegmentIndex.");
+    std::string msg("Error encountered during checkBoundarySegmentIndex.");
+    if( gridView.grid().comm().size() > 1 )
+      std::cerr << msg << std::endl;
+    else
+      DUNE_THROW(Dune::Exception, msg );
   }
 }
 
@@ -796,7 +848,7 @@ void gridcheck (Grid &g)
   enum { dimworld = Grid :: dimensionworld };
   typedef typename Grid  :: ctype ctype;
   typedef typename Grid  :: GridFamily GridFamily;
-  
+
   /*
    * Check whether we have a nonzero number of entities in each view
    * in case the grid is non-empty
@@ -807,7 +859,10 @@ void gridcheck (Grid &g)
     {
       assert(g.size(i)>0);
       for (int j=0; j<=g.maxLevel(); j++)
-        assert(g.size(j,i)>0);
+        if (g.comm().size() == 0)
+          assert(g.size(j,i)>0);
+        else
+          assert(g.size(j,i)>=0);   // in parallel this could be zero (depends on definition of maxLevel() method in parallel)
     }
   }
 
@@ -815,11 +870,11 @@ void gridcheck (Grid &g)
   // print infos
   Dune::gridinfo(g, "GridInfo");
   for(int l=0; l<g.maxLevel(); l++)
-      Dune::gridlevellist(g, l, "GridLevelInfo");
+    Dune::gridlevellist(g, l, "GridLevelInfo");
   Dune::gridleaflist(g, "GridLeafInfo");
 #endif
 
-  // type of GridInterface == GridDefaultImplementation 
+  // type of GridInterface == GridDefaultImplementation
   typedef Dune::GridDefaultImplementation<dim,dimworld,ctype,GridFamily> GridIF;
   const GridIF & gridIF = g;
   // check functionality when grid is interpreted as reference to interface
@@ -836,16 +891,23 @@ void gridcheck (Grid &g)
   zeroEntityConsistency(cg);
   assertNeighbor(g);
   assertNeighbor(cg);
+  checkFatherLevel(g);
+  checkFatherLevel(cg);
 
   // check geometries of macro level and leaf level
-  checkGeometry( g.levelView( 0 ) );
-  checkGeometry( g.leafView() );
+  checkGeometry( g.levelGridView( 0 ) );
+  checkGeometry( g.leafGridView() );
+
+  // check entity seeds
+  Dune::checkEntitySeed( g.leafView(), std::cerr );
+  for( int level = 0; level <= g.maxLevel(); ++level )
+    Dune::checkEntitySeed( g.levelGridView( level ), std::cerr );
 
   // note that for some grid this might fail
-  // then un comment this test 
+  // then un comment this test
   Dune :: checkIndexSet( g, g.leafView(), Dune :: dvverb );
   for( int level = 0; level <= g.maxLevel(); ++level )
-    Dune :: checkIndexSet( g, g.levelView( level ), Dune :: dvverb, true );
+    Dune :: checkIndexSet( g, g.levelGridView( level ), Dune :: dvverb, true );
 
   // check at least if the subId method is there
   {
@@ -864,7 +926,7 @@ void gridcheck (Grid &g)
   }
 
   if( EnableLevelIntersectionIteratorCheck< Grid >::v )
-    checkBoundarySegmentIndex( g.levelView( 0 ) );
+    checkBoundarySegmentIndex( g.levelGridView( 0 ) );
   else if( g.maxLevel() == 0 )
     checkBoundarySegmentIndex( g.leafView() );
   else
