@@ -1,4 +1,6 @@
-// $Id: test-yaspgrid.cc 8761 2013-01-19 15:09:31Z mblatt $
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
+// $Id$
 
 #include <config.h>
 
@@ -17,17 +19,17 @@ int rank;
 
 template <int dim>
 void check_yasp(bool p0=false) {
-  typedef Dune::FieldVector<int,dim> iTupel;
   typedef Dune::FieldVector<double,dim> fTupel;
-  typedef Dune::FieldVector<bool,dim> bTupel;
 
   std::cout << std::endl << "YaspGrid<" << dim << ">";
   if (p0) std::cout << " periodic\n";
   std::cout << std::endl << std::endl;
 
   fTupel Len; Len = 1.0;
-  iTupel s; s = 2; s[0] = 6;
-  bTupel p; p = false;
+  Dune::array<int,dim> s;
+  std::fill(s.begin(), s.end(), 2);
+  s[0] = 6;
+  std::bitset<dim> p;
   p[0] = p0;
   int overlap = 1;
 
@@ -40,10 +42,10 @@ void check_yasp(bool p0=false) {
   gridcheck(grid);
 
   grid.globalRefine(2);
-  
+
   gridcheck(grid);
 
-  // check communication interface 
+  // check communication interface
   checkCommunication(grid,-1,Dune::dvverb);
   for(int l=0; l<=grid.maxLevel(); ++l)
     checkCommunication(grid,l,Dune::dvverb);
@@ -54,9 +56,12 @@ void check_yasp(bool p0=false) {
   checkGeometryInFather(grid);
   // check the intersection iterator and the geometries it returns
   checkIntersectionIterator(grid);
-  // check grid adaptation interface 
+  // check grid adaptation interface
   checkAdaptRefinement(grid);
   checkPartitionType( grid.leafView() );
+
+  // test operator<<
+  std::cout << grid << std::endl;
 }
 
 int main (int argc , char **argv) {
@@ -67,7 +72,7 @@ int main (int argc , char **argv) {
 
     // get own rank
     MPI_Comm_rank(MPI_COMM_WORLD,&rank);
-#endif    
+#endif
 
     check_yasp<1>();
     //check_yasp<1>(true);
@@ -84,11 +89,11 @@ int main (int argc , char **argv) {
     std::cerr << "Generic exception!" << std::endl;
     return 2;
   }
-  
+
 #if HAVE_MPI
   // Terminate MPI
   MPI_Finalize();
 #endif
 
   return 0;
-};
+}
