@@ -1,13 +1,14 @@
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
 /** \file
     \brief A unit test for the PersistentContainer
-*/
+ */
 
 #include <config.h>
 
 #include <iostream>
-#include <cassert>
 
-#include <dune/common/mpihelper.hh>
+#include <dune/common/parallel/mpihelper.hh>
 #include <dune/grid/yaspgrid.hh>
 #if HAVE_ALUGRID
 #include <dune/grid/alugrid.hh>
@@ -36,7 +37,7 @@ bool test(GridType &grid)
   PersistentContainer<GridType,DataType> container0(grid,0);
   PersistentContainer<GridType,DataType> container1(grid,1);
   PersistentContainer<GridType,DataType> container2(grid,2);
-  
+
   typedef typename GridType::LeafGridView GridView;
   const GridView &view = grid.leafView();
   typedef typename GridView::template Codim<0>::Iterator EIterator;
@@ -46,25 +47,25 @@ bool test(GridType &grid)
     for(EIterator eit = view.template begin<0>(); eit != eend; ++eit)
     {
       container0[*eit] = eit->geometry().center();
-      const Dune::GenericReferenceElement< typename GridType::ctype, GridType::dimension > &refElement
-          = Dune::GenericReferenceElements< typename GridType::ctype, GridType::dimension >::general( eit->type() );
-      for (int i=0;i<eit->template count<1>();++i)
+      const Dune::ReferenceElement< typename GridType::ctype, GridType::dimension > &refElement
+        = Dune::ReferenceElements< typename GridType::ctype, GridType::dimension >::general( eit->type() );
+      for (int i=0; i<eit->template count<1>(); ++i)
         container1(*eit,i) = eit->geometry().global( refElement.position(i,1) );
-      for (int i=0;i<eit->template count<2>();++i)
+      for (int i=0; i<eit->template count<2>(); ++i)
         container2(*eit,i) = eit->geometry().global( refElement.position(i,2) );
     }
   }
 
   grid.globalRefine(1);
-  container0.update();
-  container1.update();
-  container2.update();
+  container0.resize();
+  container1.resize();
+  container2.resize();
 
   {
     const EIterator &eend = view.template end<0>();
     for(EIterator eit = view.template begin<0>(); eit != eend; ++eit)
     {
-      if (container0[*eit].used == true) 
+      if (container0[*eit].used == true)
       {
         std::cout << "ERROR: a new element is marked as 'used' in the container - stop testing" << std::endl;
         ret = false;
@@ -74,7 +75,7 @@ bool test(GridType &grid)
       while ( !container0[*up].used )
       {
         if (up->level() == 0)
-        {  
+        {
           std::cout << "ERROR: could not find a father element in container - stop testing" << std::endl;
           ret = false;
           break;
@@ -87,16 +88,16 @@ bool test(GridType &grid)
         ret = false;
         break;
       }
-      const Dune::GenericReferenceElement< typename GridType::ctype, GridType::dimension > &refElement
-          = Dune::GenericReferenceElements< typename GridType::ctype, GridType::dimension >::general( eit->type() );
-      for (int i=0;i<eit->template count<1>();++i)
+      const Dune::ReferenceElement< typename GridType::ctype, GridType::dimension > &refElement
+        = Dune::ReferenceElements< typename GridType::ctype, GridType::dimension >::general( eit->type() );
+      for (int i=0; i<eit->template count<1>(); ++i)
         if ( ( container1(*up,i).coord - up->geometry().global( refElement.position(i,1) ) ).two_norm() > 1e-8 )
         {
           std::cout << "ERROR: wrong data stored in container1 - stop testing" << std::endl;
           ret = false;
           break;
         }
-      for (int i=0;i<eit->template count<2>();++i)
+      for (int i=0; i<eit->template count<2>(); ++i)
         if ( ( container2(*up,i).coord - up->geometry().global( refElement.position(i,2) ) ).two_norm() > 1e-8 )
         {
           std::cout << "ERROR: wrong data stored in container1 - stop testing" << std::endl;
@@ -110,42 +111,43 @@ bool test(GridType &grid)
 
 int main (int argc , char **argv)
 try {
-  
-    // this method calls MPI_Init, if MPI is enabled
-    MPIHelper::instance(argc,argv);
-    
-    // /////////////////////////////////////////////////////////////////////////////
-    //   Test YaspGrid
-    // /////////////////////////////////////////////////////////////////////////////
-    {
-      typedef YaspGrid<2> GridType;
-      Dune::FieldVector<double,2> Len; Len = 1.0;
-      Dune::FieldVector<int,2> s; s = 2; s[0] = 6;
-      Dune::FieldVector<bool,2> p; p = false;
-      int overlap = 1;
-      GridType grid(Len,s,p,overlap);
-      std::cout << "Testing YaspGrid" << std::endl;
-      test(grid);
-    }
- 
-#if HAVE_ALUGRID
-    {
-      typedef ALUCubeGrid<2,2> GridType;
-      array<unsigned int,2> elements2d;
-      elements2d.fill(4);
-      shared_ptr<GridType> grid = StructuredGridFactory<GridType>::createCubeGrid(FieldVector<double,2>(0),
-                                                                                  FieldVector<double,2>(1), elements2d);
-      std::cout << "Testing ALUGrid" << std::endl;
-      test(*grid);
-    }
-#endif
-                                                                                   
-    return 0;
 
- } catch (Exception &e) {
-    std::cerr << e << std::endl;
-    return 1;
- } catch (...) {
-    std::cerr << "Generic exception!" << std::endl;
-    return 2;
- }
+  // this method calls MPI_Init, if MPI is enabled
+  MPIHelper::instance(argc,argv);
+
+  // /////////////////////////////////////////////////////////////////////////////
+  //   Test YaspGrid
+  // /////////////////////////////////////////////////////////////////////////////
+  {
+    typedef YaspGrid<2> GridType;
+    Dune::FieldVector<double,2> Len; Len = 1.0;
+    Dune::array<int,2> s = { {2, 6} };
+    std::bitset<2> p;
+    int overlap = 1;
+    GridType grid(Len,s,p,overlap);
+    std::cout << "Testing YaspGrid" << std::endl;
+    test(grid);
+  }
+
+#if HAVE_ALUGRID
+  {
+    typedef Dune::ALUGrid<2, 2, cube, nonconforming> GridType;
+    array<unsigned int,2> elements2d;
+    elements2d.fill(4);
+    shared_ptr<GridType> grid = StructuredGridFactory<GridType>::createCubeGrid(FieldVector<double,2>(0),
+                                                                                FieldVector<double,2>(1), elements2d);
+    std::cout << "Testing ALUGrid" << std::endl;
+    test(*grid);
+  }
+#endif
+
+  return 0;
+
+}
+catch (Exception &e) {
+  std::cerr << e << std::endl;
+  return 1;
+} catch (...) {
+  std::cerr << "Generic exception!" << std::endl;
+  return 2;
+}
