@@ -1,9 +1,9 @@
-// -*- tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
-// vi: set et ts=8 sw=2 sts=2:
+// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+// vi: set et ts=4 sw=2 sts=2:
 
 /** \file
     \brief A unit test for the VertexOrder classes
-*/
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -21,7 +21,7 @@
 #include <dune/common/fvector.hh>
 #include <dune/common/shared_ptr.hh>
 #include <dune/common/typetraits.hh>
-#include <dune/common/mpihelper.hh>
+#include <dune/common/parallel/mpihelper.hh>
 
 #include <dune/geometry/referenceelements.hh>
 #include <dune/geometry/generalvertexorder.hh>
@@ -36,8 +36,12 @@
 #define USE_ALUGRID_SURFACE_2D
 #endif
 
-void fail(int &result) { result = 1; }
-void pass(int &result) { if(result == 77) result = 0; }
+void fail(int &result) {
+  result = 1;
+}
+void pass(int &result) {
+  if(result == 77) result = 0;
+}
 
 //! test consistency on one dimension and element
 template<std::size_t mydim, class VertexOrder>
@@ -46,8 +50,8 @@ void testElementDim(const Dune::integral_constant<std::size_t, mydim>&,
 {
   static const std::size_t dim = VertexOrder::dimension;
   static const std::size_t codim = dim - mydim;
-  const Dune::GenericReferenceElement<double, dim> &refelem
-    = Dune::GenericReferenceElements<double, dim>::general(vo.type());
+  const Dune::ReferenceElement<double, dim> &refelem
+    = Dune::ReferenceElements<double, dim>::general(vo.type());
 
   std::vector<typename VertexOrder::Index> subOrder, tmp;
   for(int subentity = 0; subentity < refelem.size(codim); ++subentity)
@@ -79,14 +83,14 @@ void testElementInterdim(const Dune::integral_constant<std::size_t, mydim>&,
 
   static const std::size_t dim = VertexOrder::dimension;
   static const std::size_t codim = dim - mydim;
-  const Dune::GenericReferenceElement<double, dim> &refelem =
-    Dune::GenericReferenceElements<double, dim>::general(vo.type());
+  const Dune::ReferenceElement<double, dim> &refelem =
+    Dune::ReferenceElements<double, dim>::general(vo.type());
 
   std::vector<typename VertexOrder::Index> subOrder, tmp, subsubOrder;
   for(int subentity = 0; subentity < refelem.size(codim); ++subentity)
   {
-    const Dune::GenericReferenceElement<double, mydim> &subrefelem =
-      Dune::GenericReferenceElements<double, mydim>::
+    const Dune::ReferenceElement<double, mydim> &subrefelem =
+      Dune::ReferenceElements<double, mydim>::
       general(refelem.type(subentity, codim));
 
     subOrder.assign(vo.begin(codim, subentity), vo.end(codim, subentity));
@@ -105,7 +109,7 @@ void testElementInterdim(const Dune::integral_constant<std::size_t, mydim>&,
       if(!(subsubOrder == tmp))
         DUNE_THROW(Dune::Exception, "Codim " << codim << ": Subentity " <<
                    subentity << ": Sub-Codim " << codim+1 <<": Sub-Subentity "
-                   << subsubentity << ": Vertex numbering from subentity does "
+                                             << subsubentity << ": Vertex numbering from subentity does "
                    "not match vertex numbering from sub-subentity.");
     }
   }
@@ -124,10 +128,10 @@ void testNeighborDim(const Dune::integral_constant<std::size_t, mydim>&,
 {
   static const std::size_t dim = VertexOrder::dimension;
   static const std::size_t codim = dim - mydim;
-  const Dune::GenericReferenceElement<double, dim> &refelem_s =
-    Dune::GenericReferenceElements<double, dim>::general(vo_s.type());
-  const Dune::GenericReferenceElement<double, dim> &refelem_n =
-    Dune::GenericReferenceElements<double, dim>::general(vo_n.type());
+  const Dune::ReferenceElement<double, dim> &refelem_s =
+    Dune::ReferenceElements<double, dim>::general(vo_s.type());
+  const Dune::ReferenceElement<double, dim> &refelem_n =
+    Dune::ReferenceElements<double, dim>::general(vo_n.type());
 
   std::size_t index_s = is.indexInInside();
   std::size_t index_n = is.indexInOutside();
@@ -154,7 +158,7 @@ void testNeighborDim(const Dune::integral_constant<std::size_t, mydim>&,
       refelem_s.subEntity(index_s, 1, subentity_s, codim);
     DomainW subpos = geo_s.global(refelem_s.position(subindex_s, codim));
     int subentity_n = 0;
-    std::size_t subindex_n;
+    std::size_t subindex_n = 0;
     for(; subentity_n < refelem_n.size(index_n, 1, codim); ++subentity_n) {
       subindex_n = refelem_n.subEntity(index_n, 1, subentity_n, codim);
       if((subpos - geo_n.global(refelem_s.position(subindex_n, codim))
@@ -220,7 +224,7 @@ void testVertexOrder(const GV& gv, const VertexOrderFactory &voFactory,
 
   static const std::size_t dim = GV::dimension;
   typedef typename VertexOrderFactory::template VertexOrder<dim>::type
-    VertexOrder;
+  VertexOrder;
   typedef typename GV::template Codim<0>::Iterator EIterator;
   typedef typename GV::IntersectionIterator IIterator;
 
@@ -235,7 +239,7 @@ void testVertexOrder(const GV& gv, const VertexOrderFactory &voFactory,
         if(iit->neighbor()) {
           VertexOrder vo_n = voFactory.make(*iit->outside());
           testNeighbor(Dune::integral_constant<std::size_t,
-                                               dim-(codim==0?1:codim)>(),
+                           dim-(codim==0 ? 1 : codim)>(),
                        vo, vo_n, *iit);
         }
     }
@@ -257,7 +261,7 @@ void testVertexOrderByIdSimplices(int &result) {
   std::fill(elements.begin(), elements.end(), 4);
 
   Dune::shared_ptr<Grid> gridp = Dune::StructuredGridFactory<Grid>::
-    createSimplexGrid(Domain(0), Domain(1), elements);
+                                 createSimplexGrid(Domain(0), Domain(1), elements);
 
   typedef typename Grid::GlobalIdSet IdSet;
   typedef Dune::VertexOrderByIdFactory<IdSet> VOFactory;
@@ -277,7 +281,7 @@ void testVertexOrderByIdCubes(int &result) {
   std::fill(elements.begin(), elements.end(), 4);
 
   Dune::shared_ptr<Grid> gridp = Dune::StructuredGridFactory<Grid>::
-    createCubeGrid(Domain(0), Domain(1), elements);
+                                 createCubeGrid(Domain(0), Domain(1), elements);
 
   typedef typename Grid::GlobalIdSet IdSet;
   typedef Dune::VertexOrderByIdFactory<IdSet> VOFactory;
@@ -287,84 +291,85 @@ void testVertexOrderByIdCubes(int &result) {
 }
 
 int main (int argc , char **argv)
-  try {
-  
-    // this method calls MPI_Init, if MPI is enabled
-    Dune::MPIHelper::instance(argc,argv);
-    
-    int result = 77;
+try {
 
-    //////////////////////////////////////////////////////////////////////
-    //   Test 1d grids
-    //////////////////////////////////////////////////////////////////////
+  // this method calls MPI_Init, if MPI is enabled
+  Dune::MPIHelper::instance(argc,argv);
 
-    std::cout << "= Testding 1D" << std::endl;
+  int result = 77;
 
-    std::cout << "== Testing OneDGrid" << std::endl;
-    testVertexOrderByIdCubes<Dune::OneDGrid>(result);
+  //////////////////////////////////////////////////////////////////////
+  //   Test 1d grids
+  //////////////////////////////////////////////////////////////////////
 
-    //////////////////////////////////////////////////////////////////////
-    //   Test 2d grids
-    //////////////////////////////////////////////////////////////////////
+  std::cout << "= Testing 1D" << std::endl;
 
-    std::cout << "= Testding 2D" << std::endl;
+  std::cout << "== Testing OneDGrid" << std::endl;
+  testVertexOrderByIdCubes<Dune::OneDGrid>(result);
+
+  //////////////////////////////////////////////////////////////////////
+  //   Test 2d grids
+  //////////////////////////////////////////////////////////////////////
+
+  std::cout << "= Testing 2D" << std::endl;
 
 #if HAVE_UG
-    std::cout << "== Testing UGGrid<2> with simplices" << std::endl;
-    testVertexOrderByIdCubes<Dune::UGGrid<2> >(result);
-    std::cout << "== Testing UGGrid<2> with cubes" << std::endl;
-    testVertexOrderByIdSimplices<Dune::UGGrid<2> >(result);
+  std::cout << "== Testing UGGrid<2> with simplices" << std::endl;
+  testVertexOrderByIdCubes<Dune::UGGrid<2> >(result);
+  std::cout << "== Testing UGGrid<2> with cubes" << std::endl;
+  testVertexOrderByIdSimplices<Dune::UGGrid<2> >(result);
 #endif // HAVE_UG
 
 #if HAVE_ALUGRID
-    std::cout << "== Testing ALUConformGrid<2,2> with simplices" << std::endl;
-    testVertexOrderByIdSimplices<Dune::ALUConformGrid<2, 2> >(result);
+  std::cout << "== Testing ALUGrid<2,2,simplex,conforming> with simplices" << std::endl;
+  testVertexOrderByIdSimplices<Dune::ALUGrid<2, 3, Dune::simplex, Dune::conforming> >(result);
 #ifdef USE_ALUGRID_SURFACE_2D
-    std::cout << "== Testing ALUConformGrid<2,3> with simplices" << std::endl;
-    testVertexOrderByIdSimplices<Dune::ALUConformGrid<2, 3> >(result);
+  std::cout << "== Testing ALUGrid<2,3,simplex,conforming> with simplices" << std::endl;
+  testVertexOrderByIdSimplices<Dune::ALUGrid<2, 3, Dune::simplex, Dune::conforming> >(result);
 #endif // USE_ALUGRID_SURFACE_2D
 
 #ifdef USE_ALUGRID_SURFACE_2D
-    std::cout << "== Testing ALUCubeGrid<2,2> with cubes" << std::endl;
-    testVertexOrderByIdCubes<Dune::ALUCubeGrid<2, 2> >(result);
-    std::cout << "== Testing ALUCubeGrid<2,3> with cubes" << std::endl;
-    testVertexOrderByIdCubes<Dune::ALUCubeGrid<2, 3> >(result);
+  std::cout << "== Testing Dune::ALUGrid<2,2,cube,nonconforming> with cubes" << std::endl;
+  testVertexOrderByIdCubes<Dune::ALUGrid<2, 3, Dune::cube, Dune::nonconforming> >(result);
+  std::cout << "== Testing Dune::ALUGrid<2,3,cube,nonconforming> with cubes" << std::endl;
+  testVertexOrderByIdCubes<Dune::ALUGrid<2, 3, Dune::cube, Dune::nonconforming> >(result);
 #endif // USE_ALUGRID_SURFACE_2D
 
-    std::cout << "== Testing ALUSimplexGrid<2,2> with simplices" << std::endl;
-    testVertexOrderByIdSimplices<Dune::ALUSimplexGrid<2, 2> >(result);
+  std::cout << "== Testing ALUGrid<2,2,simplex,nonconforming> with simplices with simplices" << std::endl;
+  testVertexOrderByIdSimplices<Dune::ALUGrid<2, 2, Dune::simplex, Dune::nonconforming> >(result);
 #ifdef USE_ALUGRID_SURFACE_2D
-    std::cout << "== Testing ALUSimplexGrid<2,3> with simplices" << std::endl;
-    testVertexOrderByIdSimplices<Dune::ALUSimplexGrid<2, 3> >(result);
+  std::cout << "== Testing ALUGrid<2,3,simplex,nonconforming> with simplices with simplices" << std::endl;
+  testVertexOrderByIdSimplices<Dune::ALUGrid<2, 3, Dune::simplex, Dune::nonconforming> >(result);
 #endif // USE_ALUGRID_SURFACE_2D
 #endif // HAVE_ALUGRID
 
-    //////////////////////////////////////////////////////////////////////
-    //   Test 3d grids
-    //////////////////////////////////////////////////////////////////////
+  //////////////////////////////////////////////////////////////////////
+  //   Test 3d grids
+  //////////////////////////////////////////////////////////////////////
 
-    std::cout << "= Testding 3D" << std::endl;
+  std::cout << "= Testing 3D" << std::endl;
 
 #if HAVE_UG
-    std::cout << "== Testing UGGrid<3> with simplices" << std::endl;
-    testVertexOrderByIdCubes<Dune::UGGrid<3> >(result);
-    std::cout << "== Testing UGGrid<3> with cubes" << std::endl;
-    testVertexOrderByIdSimplices<Dune::UGGrid<3> >(result);
+  std::cout << "== Testing UGGrid<3> with simplices" << std::endl;
+  testVertexOrderByIdCubes<Dune::UGGrid<3> >(result);
+  std::cout << "== Testing UGGrid<3> with cubes" << std::endl;
+  testVertexOrderByIdSimplices<Dune::UGGrid<3> >(result);
 #endif // HAVE_UG
 
 #if HAVE_ALUGRID
-    std::cout << "== Testing ALUCubeGrid<3,3> with cubes" << std::endl;
-    testVertexOrderByIdCubes<Dune::ALUCubeGrid<3, 3> >(result);
-    std::cout << "== Testing ALUSimplexGrid<3,3> with simplices" << std::endl;
-    testVertexOrderByIdSimplices<Dune::ALUSimplexGrid<3, 3> >(result);
+  std::cout << "== Testing Dune::ALUGrid<3,3,cube,nonconforming> with cubes" << std::endl;
+  testVertexOrderByIdCubes<Dune::ALUGrid<3, 3, Dune::cube, Dune::nonconforming> >(result);
+  std::cout << "== Testing ALUGrid<3,3,simplex,nonconforming> with simplices" << std::endl;
+  testVertexOrderByIdSimplices<Dune::ALUGrid<3, 3, Dune::simplex, Dune::nonconforming> >(result);
 #endif // HAVE_ALUGRID
 
-    return result;
+  return result;
 
-  } catch (const Dune::Exception &e) {
-    std::cerr << e << std::endl;
-    throw;
-  } catch (...) {
-    std::cerr << "Generic exception!" << std::endl;
-    throw;
-  }
+}
+catch (const Dune::Exception &e) {
+  std::cerr << e << std::endl;
+  throw;
+} catch (...) {
+  std::cerr << "Generic exception!" << std::endl;
+  throw;
+}
